#
# Author: Pearu Peterson, March 2002
#
# w/ additions by Travis Oliphant, March 2002

__all__ = ['solve','inv','det','lstsq','norm','pinv','pinv2',
           'tri','tril','triu','toeplitz','hankel','lu_solve',
           'cho_solve','solve_banded','LinAlgError','kron',
           'all_mat']

#from blas import get_blas_funcs
from lapack import get_lapack_funcs
from flinalg import get_flinalg_funcs
from scipy_base import asarray,zeros,sum,NewAxis,greater_equal,subtract,arange,\
     conjugate,ravel,r_,mgrid,take,ones,dot,transpose,diag,sqrt,add,real
import Matrix
import scipy_base
from scipy_base import asarray_chkfinite
import calc_lwork

class LinAlgError(Exception):
    pass

def lu_solve((lu, piv), b, trans=0, overwrite_b=0):
    """ lu_solve((lu, piv), b, trans=0, overwrite_b=0) -> x

    Solve a system of equations given a previously factored matrix

    Inputs:

      (lu,piv) -- The factored matrix, a (the output of lu_factor)
      b        -- a set of right-hand sides
      trans    -- type of system to solve:
                  0 : a   * x = b   (no transpose)
                  1 : a^T * x = b   (transpose) 
                  2   a^H * x = b   (conjugate transpose)

    Outputs:

       x -- the solution to the system
    """
    b1 = asarray_chkfinite(b)
    overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
    if lu.shape[0] != b1.shape[0]:
        raise ValuError, "incompatible dimensions."
    getrs, = get_lapack_funcs(('getrs',),(lu,b1))
    x,info = getrs(lu,piv,b1,trans=trans,overwrite_b=overwrite_b)
    if info==0:
        return x
    raise ValueError,\
          'illegal value in %-th argument of internal gesv|posv'%(-info)

def cho_solve((c, lower), b, overwrite_b=0):
    """ cho_solve((c, lower), b, overwrite_b=0) -> x

    Solve a system of equations given a previously cholesky factored matrix

    Inputs:

      (c,lower) -- The factored matrix, a (the output of cho_factor)
      b        -- a set of right-hand sides

    Outputs:

       x -- the solution to the system a*x = b
    """
    b1 = asarray_chkfinite(b)
    overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
    if c.shape[0] != b1.shape[0]:
        raise ValuError, "incompatible dimensions."
    potrs, = get_lapack_funcs(('potrs',),(c,b1))
    x,info = potrs(c,b1,lower=lower,overwrite_b=overwrite_b)
    if info==0:
        return x
    raise ValueError,\
          'illegal value in %-th argument of internal gesv|posv'%(-info)

# Linear equations
def solve(a, b, sym_pos=0, lower=0, overwrite_a=0, overwrite_b=0,
          debug = 0):
    """ solve(a, b, sym_pos=0, lower=0, overwrite_a=0, overwrite_b=0) -> x

    Solve a linear system of equations a * x = b for x.

    Inputs:

      a -- An N x N matrix.
      b -- An N x nrhs matrix or N vector.
      sym_pos -- Assume a is symmetric and positive definite.
      lower -- Assume a is lower triangular, otherwise upper one.
               Only used if sym_pos is true.
      overwrite_y - Discard data in y, where y is a or b.

    Outputs:

      x -- The solution to the system a * x = b
    """
    a1, b1 = map(asarray_chkfinite,(a,b))
    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
        raise ValueError, 'expected square matrix'
    if a1.shape[0] != b1.shape[0]:
        raise ValueError, 'incompatible dimensions'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
    if debug:
        print 'solve:overwrite_a=',overwrite_a
        print 'solve:overwrite_b=',overwrite_b
    if sym_pos:
        posv, = get_lapack_funcs(('posv',),(a1,b1),debug=debug)
        c,x,info = posv(a1,b1,
                        lower = lower,
                        overwrite_a=overwrite_a,
                        overwrite_b=overwrite_b)
    else:
        gesv, = get_lapack_funcs(('gesv',),(a1,b1))
        lu,piv,x,info = gesv(a1,b1,
                             overwrite_a=overwrite_a,
                             overwrite_b=overwrite_b)
        
    if info==0:
        return x
    if info>0:
        raise LinAlgError, "singular matrix"
    raise ValueError,\
          'illegal value in %-th argument of internal gesv|posv'%(-info)

def solve_banded((l,u), ab, b, overwrite_ab=0, overwrite_b=0,
          debug = 0):
    """ solve_banded((l,u), ab, b, overwrite_ab=0, overwrite_b=0) -> x

    Solve a linear system of equations a * x = b for x where
    a is a banded matrix stored in diagonal orded form

     *   *     a1u
     
     *  a12 a23 ...
    a11 a22 a33 ...
    a21 a32 a43 ...
    .
    al1 ..         *

    Inputs:

      (l,u) -- number of non-zero lower and upper diagonals, respectively.
      a -- An N x (l+u+1) matrix.
      b -- An N x nrhs matrix or N vector.
      overwrite_y - Discard data in y, where y is ab or b.

    Outputs:

      x -- The solution to the system a * x = b
    """
    a1, b1 = map(asarray_chkfinite,(ab,b))
    overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))

    gbsv, = get_lapack_funcs(('gbsv',),(a1,b1))
    a2 = zeros((2*l+u+1,a1.shape[1]),gbsv.typecode)
    a2[l:,:] = a1 
    lu,piv,x,info = gbsv(l,u,a2,b1,
                         overwrite_ab=1,
                         overwrite_b=overwrite_b)
    if info==0:
        return x
    if info>0:
        raise LinAlgError, "singular matrix"
    raise ValueError,\
          'illegal value in %-th argument of internal gbsv'%(-info)


# matrix inversion
def inv(a, overwrite_a=0):
    """ inv(a, overwrite_a=0) -> a_inv

    Return inverse of square matrix a.
    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    #XXX: I found no advantage or disadvantage of using finv.
##     finv, = get_flinalg_funcs(('inv',),(a1,))
##     if finv is not None:
##         a_inv,info = finv(a1,overwrite_a=overwrite_a)
##         if info==0:
##             return a_inv
##         if info>0: raise LinAlgError, "singular matrix"
##         if info<0: raise ValueError,\
##            'illegal value in %-th argument of internal inv.getrf|getri'%(-info)
    getrf,getri = get_lapack_funcs(('getrf','getri'),(a1,))
    #XXX: C ATLAS versions of getrf/i have rowmajor=1, this could be
    #     exploited for further optimization. But it will be probably
    #     a mess. So, a good testing site is required before trying
    #     to do that.
    if getrf.module_name[:7]=='clapack'!=getri.module_name[:7]:
        # ATLAS 3.2.1 has getrf but not getri.
        lu,piv,info = getrf(transpose(a1),
                            rowmajor=0,overwrite_a=overwrite_a)
        lu = transpose(lu)
    else:
        lu,piv,info = getrf(a1,overwrite_a=overwrite_a)
    if info==0:
        if getri.module_name[:7] == 'flapack':
            lwork = calc_lwork.getri(getri.prefix,a1.shape[0])
            lwork = lwork[1]
            # XXX: the following line fixes curious SEGFAULT when
            # benchmarking 500x500 matrix inverse. This seems to
            # be a bug in LAPACK ?getri routine because if lwork is
            # minimal (when using lwork[0] instead of lwork[1]) then
            # all tests pass. Further investigation is required if
            # more such SEGFAULTs occur.
            lwork = int(1.01*lwork)
            inv_a,info = getri(lu,piv,
                               lwork=lwork,overwrite_lu=1)
        else: # clapack
            inv_a,info = getri(lu,piv,overwrite_lu=1)
    if info>0: raise LinAlgError, "singular matrix"
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal getrf|getri'%(-info)
    return inv_a


## matrix and Vector norm
import decomp
def norm(x, ord=2):
    """ norm(x, ord=2) -> n

    Matrix and vector norm.

    Inputs:

      x -- a rank-1 (vector) or rank-2 (matrix) array
      ord -- the order of norm.

     Comments:

       For vectors ord can be any real number including Inf or -Inf.
         ord = Inf, computes the maximum of the magnitudes
         ord = -Inf, computes minimum of the magnitudes
         ord is finite, computes sum(abs(x)**ord)**(1.0/ord)

       For matrices ord can only be + or - 1, 2, Inf.
         ord = 2 computes the largest singular value
         ord = -2 computes the smallest singular value
         ord = 1 computes the largest column sum of absolute values
         ord = -1 computes the smallest column sum of absolute values
         ord = Inf computes the largest row sum of absolute values
         ord = -Inf computes the smallest row sum of absolute values
         ord = 'fro' computes the frobenius norm sqrt(sum(diag(X.H * X)))
    """
    x = asarray_chkfinite(x)
    nd = len(x.shape)
    Inf = scipy_base.Inf
    if nd == 1:
        if ord == Inf:
            return scipy_base.amax(abs(x))
        elif ord == -Inf:
            return scipy_base.amin(abs(x))
        else:
            return scipy_base.sum(abs(x)**ord)**(1.0/ord)
    elif nd == 2:
        if ord == 2:
            return scipy_base.amax(decomp.svd(x,compute_uv=0))
        elif ord == -2:
            return scipy_base.amin(decomp.svd(x,compute_uv=0))
        elif ord == 1:
            return scipy_base.amax(scipy_base.sum(abs(x)))
        elif ord == Inf:
            return scipy_base.amax(scipy_base.sum(abs(x),axis=1))
        elif ord == -1:
            return scipy_base.amin(scipy_base.sum(abs(x)))
        elif ord == -Inf:
            return scipy_base.amin(scipy_base.sum(abs(x),axis=1))
        elif ord in ['fro','f']:
            val = real((conjugate(x)*x).flat)
            return sqrt(add.reduce(val))
        else:
            raise ValueError, "Invalid norm order for matrices."
    else:
        raise ValueError, "Improper number of dimensions to norm."

### Determinant

def det(a, overwrite_a=0):
    """ det(a, overwrite_a=0) -> d

    Return determinant of a square matrix.
    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    fdet, = get_flinalg_funcs(('det',),(a1,))
    a_det,info = fdet(a1,overwrite_a=overwrite_a)
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal det.getrf'%(-info)
    return a_det

### Linear Least Squares

def lstsq(a, b, cond=None, overwrite_a=0, overwrite_b=0):
    """ lstsq(a, b, cond=None, overwrite_a=0, overwrite_b=0) -> x,resids,rank,s

    Return least-squares solution of a * x = b.

    Inputs:

      a -- An M x N matrix.
      b -- An M x nrhs matrix or M vector.
      cond -- Used to determine effective rank of a.

    Outputs:

      x -- The solution (N x nrhs matrix) to the minimization problem:
                  2-norm(| b - a * x |) -> min
      resids -- The residual sum-of-squares for the solution matrix x
                (only if M>N and rank==N).
      rank -- The effective rank of a.
      s -- Singular values of a in decreasing order. The condition number
           of a is abs(s[0]/s[-1]).
    """
    a1, b1 = map(asarray_chkfinite,(a,b))
    if len(a1.shape) != 2:
        raise ValueError, 'expected matrix'
    m,n = a1.shape
    if len(b1.shape)==2: nrhs = b1.shape[1]
    else: nrhs = 1
    if m != b1.shape[0]:
        raise ValueError, 'incompatible dimensions'
    gelss, = get_lapack_funcs(('gelss',),(a1,b1))
    if n>m:
        # need to extend b matrix as it will be filled with
        # a larger solution matrix
        b2 = zeros((n,nrhs),gelss.typecode)
        if len(b1.shape)==2: b2[:m,:] = b1
        else: b2[:m,0] = b1
        b1 = b2
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
    if gelss.module_name[:7] == 'flapack':
        lwork = calc_lwork.gelss(gelss.prefix,m,n,nrhs)[1]
        v,x,s,rank,info = gelss(a1,b1,cond = cond,
                                lwork = lwork,
                                overwrite_a = overwrite_a,
                                overwrite_b = overwrite_b)
    else:
        raise NotImplementedError,'calling gelss from %s' % (gelss.module_name)
    if info>0: raise LinAlgError, "SVD did not converge in Linear Least Squares"
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal gelss'%(-info)
    resids = asarray([],x.typecode())
    if n<m:
        x1 = x[:n]
        if rank==n: resids = sum(x[n:]**2)
        x = x1
    return x,resids,rank,s


def pinv(a, cond=None):
    """ pinv(a, cond=None) -> a_pinv

    Compute generalized inverse of A using least-squares solver.
    """
    a = asarray_chkfinite(a)
    t = a.typecode()
    b = scipy_base.identity(a.shape[0],t)
    return lstsq(a, b, cond=cond)[0]


eps = scipy_base.limits.double_epsilon
feps = scipy_base.limits.float_epsilon
_array_precision = {'f': 0, 'd': 1, 'F': 0, 'D': 1}
def pinv2(a, cond=None):
    """ pinv2(a, cond=None) -> a_pinv

    Compute the generalized inverse of A using svd.
    """
    a = asarray_chkfinite(a)
    u, s, vh = decomp.svd(a)
    t = u.typecode()
    if cond in [None,-1]:
        cond = {0: feps*1e3, 1: eps*1e6}[_array_precision[t]]
    m,n = a.shape
    cutoff = cond*scipy_base.maximum.reduce(s)
    psigma = zeros((m,n),t)
    for i in range(len(s)):
        if s[i] > cutoff:
            psigma[i,i] = 1.0/conjugate(s[i])
    #XXX: use lapack/blas routines for dot
    return transpose(conjugate(dot(dot(u,psigma),vh)))

#-----------------------------------------------------------------------------
# matrix construction functions
#-----------------------------------------------------------------------------

def tri(N, M=None, k=0, typecode=None):
    """ returns a N-by-M matrix where all the diagonals starting from 
        lower left corner up to the k-th are all ones.
    """
    if M is None: M = N
    if type(M) == type('d'): 
        typecode = M
        M = N
    m = greater_equal(subtract.outer(arange(N), arange(M)),-k)
    if typecode is None:
        return m
    else:
        return m.astype(typecode)

def tril(m, k=0):
    """ returns the elements on and below the k-th diagonal of m.  k=0 is the
        main diagonal, k > 0 is above and k < 0 is below the main diagonal.
    """
    svsp = m.spacesaver()
    m = asarray(m,savespace=1)
    out = tri(m.shape[0], m.shape[1], k=k, typecode=m.typecode())*m
    out.savespace(svsp)
    return out

def triu(m, k=0):
    """ returns the elements on and above the k-th diagonal of m.  k=0 is the
        main diagonal, k > 0 is above and k < 0 is below the main diagonal.
    """
    svsp = m.spacesaver()
    m = asarray(m,savespace=1)
    out = (1-tri(m.shape[0], m.shape[1], k-1, m.typecode()))*m
    out.savespace(svsp)
    return out

def toeplitz(c,r=None):
    """ Construct a toeplitz matrix (i.e. a matrix with constant diagonals).

        Description:
    
           toeplitz(c,r) is a non-symmetric Toeplitz matrix with c as its first
           column and r as its first row.
    
           toeplitz(c) is a symmetric (Hermitian) Toeplitz matrix (r=c). 
    
        See also: hankel
    """
    isscalar = scipy_base.isscalar
    if isscalar(c) or isscalar(r):
        return c   
    if r is None:
        r = c
        r[0] = conjugate(r[0])
        c = conjugate(c)
    r,c = map(asarray_chkfinite,(r,c))
    r,c = map(ravel,(r,c))
    rN,cN = map(len,(r,c))
    if r[0] != c[0]:
        print "Warning: column and row values don't agree; column value used."
    vals = r_[r[rN-1:0:-1], c]
    cols = mgrid[0:cN]
    rows = mgrid[rN:0:-1]
    indx = cols[:,NewAxis]*ones((1,rN)) + \
           rows[NewAxis,:]*ones((cN,1)) - 1
    return take(vals, indx)


def hankel(c,r=None):
    """ Construct a hankel matrix (i.e. matrix with constant anti-diagonals).
    
        Description:
    
          hankel(c,r) is a Hankel matrix whose first column is c and whose
          last row is r.
    
          hankel(c) is a square Hankel matrix whose first column is C.
          Elements below the first anti-diagonal are zero.
    
        See also:  toeplitz
    """
    isscalar = scipy_base.isscalar
    if isscalar(c) or isscalar(r):
        return c   
    if r is None:
        r = zeros(len(c))
    elif r[0] != c[-1]:
        print "Warning: column and row values don't agree; column value used."
    r,c = map(asarray_chkfinite,(r,c))
    r,c = map(ravel,(r,c))
    rN,cN = map(len,(r,c))
    vals = r_[c, r[1:rN]]
    cols = mgrid[1:cN+1]
    rows = mgrid[0:rN]
    indx = cols[:,NewAxis]*ones((1,rN)) + \
           rows[NewAxis,:]*ones((cN,1)) - 1
    return take(vals, indx)

def all_mat(*args):
    return map(Matrix.Matrix,args)

def kron(a,b):
    """kronecker product of a and b

    Kronecker product of two matrices is block matrix
    [[ a[ 0 ,0]*b, a[ 0 ,1]*b, ... , a[ 0 ,n-1]*b  ],
     [ ...                                   ...   ],
     [ a[m-1,0]*b, a[m-1,1]*b, ... , a[m-1,n-1]*b  ]]
    """
    if not a.iscontiguous():
        a = reshape(a, a.shape)
    if not b.iscontiguous():
        b = reshape(b, b.shape)
    o = outerproduct(a,b)
    o.shape = a.shape + b.shape
    return concatenate(concatenate(o, axis=1), axis=1)




import os
import sys
import time
from distutils.command.config    import config
from scipy_distutils.system_info import get_info

#------------------------------------------------------------------------
# Configuration
#------------------------------------------------------------------------

class config_pygist(config):

    def __init__(self,local_path,config_path):
        from distutils.dist import Distribution
        config.__init__(self, Distribution())
        self.local_path = local_path
        self.config_path = config_path
        self.config_h = os.path.join(self.config_path, "config.h")

    def _link (self, body,
               headers, include_dirs,
               libraries, library_dirs, lang):
        (src, obj) = self._compile(body, headers, include_dirs, lang)
        prog = os.path.splitext(os.path.basename(src))[0]
        self.compiler.link_executable([obj], prog,
                                      libraries=libraries,
                                      library_dirs=library_dirs)

        # This code is added to correct error
        if self.compiler.exe_extension is None:
            self.compiler.exe_extension = ''
        # End of code different from standard 
        prog = prog + self.compiler.exe_extension
        self.temp_files.append(prog)

        return (src, obj, prog)
    
    def run (self):
        fn = os.path.join(self.config_path, "Make.cfg")
        if os.path.isfile(fn) and os.path.isfile(self.config_h):
            print '*'*70
            print 'Files\n%s\n%s\n exist.' % (fn,self.config_h)
            print 'Skipping pygist configuration'\
                  ' (remove %s to force reconfiguration).' % fn
            print '*'*70
            return

        from distutils.ccompiler import new_compiler
        self.compiler = new_compiler(compiler=self.compiler,
                                     verbose=1)
        from distutils.sysconfig import customize_compiler
        customize_compiler(self.compiler)

        self.configfile = open(fn,'w')
        self.configfile.write('# Make.cfg from setup.py script ' + time.ctime() + '\n')
        if sys.platform != 'win32':
            self.configfile.write('#')
            for item in os.uname():
                self.configfile.write(' '+item)
            self.configfile.write('\n')

        self.config_toplevel()
        self.config_unix()
        self.config_x11()
        self.configfile.close()
        print 'wrote',fn

    if sys.version[:3]<'2.2':
        def try_run (self, body,
                     headers=None, include_dirs=None,
                     libraries=None, library_dirs=None,
                     lang="c"):
            """Try to compile, link to an executable, and run a program
            built from 'body' and 'headers'.  Return true on success, false
            otherwise.
            """
            from distutils.ccompiler import CompileError, LinkError,\
                 DistutilsExecError
            self._check_compiler()
            try:
                src,obj,exe=self._link(body, headers, include_dirs,
                                       libraries, library_dirs, lang)
                self.spawn([os.path.join('.',exe)])
                ok = 1
            except (CompileError, LinkError, DistutilsExecError):
                ok = 0

            self.announce(ok and "success!" or "failure.")
            self._clean()
            return ok

        def try_compile (self, body, headers=None, include_dirs=None, lang="c"):
            """Try to compile a source file built from 'body' and 'headers'.
            Return true on success, false otherwise.
            """
            from distutils.ccompiler import CompileError
            self._check_compiler()
            try:
                self._compile(body, headers, include_dirs, lang)
                ok = 1
            except CompileError:
                ok = 0

            self.announce(ok and "success!" or "failure.")
            self._clean()
            return ok


#----------------------------------------------------------------------
    def config_toplevel(self):
        print "  ============= begin top level configuration ============="

        testcode = """\
/* check whether libm is broken */
#include <math.h>
int main(int argc, char *argv[])
{
  return exp(-720.) > 1.0;  /* typically an IEEE denormal */
}
"""
        # First try without libm to workaround MSVC6.0 compiler.
        if self.try_link(testcode) and self.try_run(testcode):
            print "No libm needed."
            self.mathlib = None
        # check alternate libm for Alpha Linux (see play/unix/README.fpu)
        elif not os.environ.has_key('MATHLIB'):
            self.mathlib = 'm'
            if self.try_link(testcode,libraries=[self.mathlib]):
                if not self.try_run(testcode,libraries=[self.mathlib]):
                    if self.try_link(testcode, libraries=['cpml']):
                        self.mathlib="cpml"
                        print "WARNING - using -lcpml instead of -lm"
                    else:
                        print "WARNING - libm broken? see play/unix/README.fpu"
                        print "  if on Alpha Linux, rerun ./configure with CC='gcc -mieee'"
            else:
                raise "math library missing; rerun setup.py after setting the MATHLIB env variable"
        else:
            self.mathlib=os.environ['MATHLIB']
        libraries = []
        if self.mathlib:
            self.configfile.write('MATHLIB=-l'+self.mathlib+'\n')
            libraries.append(self.mathlib)
        # check exp10 presence, emulate otherwise
        testcode = """\
int main(int argc, char *argv[])
{
  double x=exp10(3.);
  return (x<999.999)||(x>1000.001);
}
"""
        if self.try_link(testcode,libraries=libraries):
            print "using exp10 found in libm"
            self.configfile.write("NO_EXP10=\n")
        else:
            print "libm does not contain exp10, will emulate"
            self.configfile.write("NO_EXP10=-DNO_EXP10\n")
#----------------------------------------------------------------------
    def config_unix(self):
        if sys.platform == 'win32':
            configfile = open(self.config_h,'w')
            configfile.write('\n')
            configfile.close()
            return
        # begin play/unix configuration
        print
        print "  ============= begin play/unix configuration ============="
        print
        configfile = open(self.config_h,'w')
        configfile.write('/* config.h used during config.sh script */\n')
        configfile.write('#ifndef CONFIG_SCRIPT\n')
        configfile.write('# error destroy this config.h and rerun configure script\n')
        configfile.write('#endif\n')
        configfile.close()
        configfile = open(self.config_h+'0','w')
        configfile.write('/* config.h from config.sh script ' + time.ctime() + '\n')
        configfile.write(' *')
        for item in os.uname():
            configfile.write(' '+item)
        configfile.write('\n')
        configfile.write(' */\n')
        self.fatality = 0

        self.unix_include_dirs = [os.path.abspath(os.path.dirname(self.config_h)),
                                  os.path.abspath(os.path.join(self.local_path,
                                                               'src','play'))]
        _olddir = os.getcwd()
        os.chdir(os.path.join(self.local_path, 'src','play','unix'))
        if sys.platform!='cygwin':
            self.find_time(configfile)
            self.find_wall_time(configfile)
            self.find_sigfpe(configfile)
        self.find_user_name(configfile)
        self.find_tiocgpgrp(configfile)
        self.find_cwd(configfile)
        self.find_dirent(configfile)
        self.find_poll(configfile)
        os.chdir(_olddir)

        if self.fatality:
            print "*** at least one play/unix component could not be configured"
            print "*** see configuration notes in play/unix/README.cfg"
        else:
            configfile.close()
            os.rename(self.config_h+'0',self.config_h)
            print "wrote config.h"


        print
        print "  ============== end play/unix configuration =============="
        
    def find_time(self,configfile):
        # find CPU time function (getrusage is best if present)
        testcode = """\
/* check settings of: USE_GETRUSAGE USE_TIMES */
#define CONFIG_SCRIPT
#include "timeu.c"
int
main(int argc, char *argv[])
{
  double s;
  double t = p_cpu_secs(&s);
  return 0;
}
"""
        if self.try_link("#define USE_GETRUSAGE\n"+testcode,
                         include_dirs=self.unix_include_dirs):
            print "using getrusage() (CPU timer)"
            configfile.write('#define USE_GETRUSAGE\n')
        elif self.try_link("#define USE_TIMES\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using times() (CPU timer)"
            configfile.write('#define USE_TIMES\n')
        elif self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "fallback to clock(), getrusage() and times() missing (CPU timer)"
        else:
            print "FATAL getrusage(), times(), and clock() all missing (timeu.c)"
            self.fatality = 1

    def find_wall_time(self,configfile):
        # find wall time function (gettimeofday is best if present)
        testcode = """\
/* check settings of: USE_GETTIMEOFDAY */
#define CONFIG_SCRIPT
#include "timew.c"
int
main(int argc, char *argv[])
{
  double t = p_wall_secs();
  return 0;
}
"""
        if self.try_link("#define USE_GETTIMEOFDAY\n"+testcode,
                         include_dirs=self.unix_include_dirs):
            print "using gettimeofday() (wall timer)"
            configfile.write('#define USE_GETTIMEOFDAY\n')
        elif self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "fallback to time()+difftime(), gettimeofday() missing (wall timer)"
        else:
            print "FATAL gettimeofday(), and time() or difftime() missing (timew.c)"
            self.fatality = 1

    def find_sigfpe(self,configfile):
#----------------------------------------------------------------------
# try to figure out how to get SIGFPE delivered
#----------------------------------------------------------------------
        if sys.platform=='cygwin':
            # No SIGFPE delivery on Windows.
            print 'hardwiring #define FPU_IGNORE'
            configfile.write('#define FPU_IGNORE\n')
            return

        testcode="""\
#define CONFIG_SCRIPT
#include "fputest.c"
"""

        testgcc="""\
#define CONFIG_SCRIPT
#include "config.c"
"""

        fpedef=""
        fpelib=""
        fpelibm=""
        if self.try_link("#define FPU_DIGITAL\n"+testcode,
                         include_dirs=self.unix_include_dirs):
            print "using FPU_DIGITAL (SIGFPE delivery)"
            configfile.write('#define FPU_DIGITAL\n')
            fpedef = "-DFPU_DIGITAL"
        elif self.try_link("#define FPU_AIX\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using FPU_AIX (SIGFPE delivery)"
            configfile.write('#define FPU_AIX\n')
            fpedef = "-DFPU_AIX"
        elif self.try_link("#define FPU_HPUX\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using FPU_HPUX (SIGFPE delivery)"
            configfile.write('#define FPU_HPUX\n')
            fpedef = "-DFPU_HPUX"
            fpelibm = self.mathlib
        elif self.try_link("#define FPU_SOLARIS\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using FPU_SOLARIS (SIGFPE delivery)"
            configfile.write('#define FPU_SOLARIS\n')
            fpedef="-DFPU_SOLARIS"
            # note this works under IRIX 6.3, while FPU_IRIX does not??
        elif self.try_link("#define FPU_SUN4\n"+testcode,
                           libraries=[self.mathlib],
                           include_dirs=self.unix_include_dirs):
            print "using FPU_SUN4 (-lm) (SIGFPE delivery)"
            configfile.write('#define FPU_SUN4\n')
            fpedef="-DFPU_SUN4"
            fpelibm=self.mathlib
        elif self.try_link("#define FPU_SUN4\n"+testcode,
                           libraries=["sunmath"],
                           include_dirs=self.unix_include_dirs):
            print "using FPU_SUN4 (-lsunmath) (SIGFPE delivery)"
            configfile.write('#define FPU_SUN4\n')
            fpedef="-DFPU_SUN4"
            fpelib="sunmath"
        elif self.try_link("#define FPU_IRIX\n"+testcode,
                           libraries=["fpe"],
                           include_dirs=self.unix_include_dirs):
            # FPU_SOLARIS seems to work better??
            print "using FPU_IRIX (SIGFPE delivery)"
            configfile.write('#define FPU_IRIX\n')
            fpedef="-DFPU_IRIX"
            fpelib="fpe"
        elif self.try_link("#define FPU_IRIX\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using FPU_IRIX (SIGFPE delivery), but no libfpe??"
            configfile.write('#define FPU_IRIX\n')
            fpedef="-DFPU_IRIX"
        elif self.try_link("#define FPU_MACOSX\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using FPU_MACOSX (SIGFPE delivery)"
            configfile.write('#define FPU_MACOSX\n')
            fpedef="-DFPU_MACOSX"
        elif self.try_compile("#define TEST_GCC\n"+testgcc,
                              include_dirs=self.unix_include_dirs):
            if self.try_link("#define FPU_ALPHA_LINUX\n" + testcode,
                             include_dirs=self.unix_include_dirs):
                print "using FPU_ALPHA_LINUX (SIGFPE delivery)"
                configfile.write('#define FPU_ALPHA_LINUX\n')
                fpedef="-DFPU_ALPHA_LINUX"
                print "...libm may be broken -- read play/unix/README.fpu for more"
                print "...fputest failure may not mean that pygist itself is broken"
                # CC="$CC -mfp-trap-mode=su -mtrap-precision=i"
            elif self.try_link("#define FPU_GCC_I86\n" + testcode,
                               include_dirs=self.unix_include_dirs):
                print "using FPU_GCC_I86 (SIGFPE delivery)"
                configfile.write('#define FPU_GCC_I86\n')
                fpedef="-DFPU_GCC_I86"
            elif self.try_link("#define FPU_GCC_SPARC\n" + testcode,
                               include_dirs=self.unix_include_dirs):
                print "using FPU_GCC_SPARC (SIGFPE delivery)"
                configfile.write('#define FPU_GCC_SPARC\n')
                fpedef="-DFPU_GCC_SPARC"
            elif self.try_link("#define FPU_GCC_M86K\n" + testcode,
                               include_dirs=self.unix_include_dirs):
                print "using FPU_GCC_M68K (SIGFPE delivery)"
                configfile.write('#define FPU_GCC_M68K\n')
                fpedef="-DFPU_GCC_M68K"
            elif self.try_link("#define FPU_GCC_POWERPC\n" + testcode,
                               include_dirs=self.unix_include_dirs):
                print "using FPU_GCC_POWERPC (SIGFPE delivery)"
                configfile.write('#define FPU_GCC_POWERPC\n')
                fpedef="-DFPU_GCC_POWERPC"
            elif self.try_link("#define FPU_GCC_ARM\n" + testcode,
                               include_dirs=self.unix_include_dirs):
                print "using FPU_GCC_ARM (SIGFPE delivery)"
                configfile.write('#define FPU_GCC_ARM\n')
                fpedef="-DFPU_GCC_ARM"
        elif self.try_link("#define FPU_GNU_FENV \n" + testcode,
                           libraries=[self.mathlib],
                           include_dirs=self.unix_include_dirs):
	    raise 'kutje'
            print "using FPU_GNU_FENV (SIGFPE delivery)"
            configfile.write('#define FPU_GNU_FENV\n')
            fpedef="-DFPU_GNU_FENV"
            fpelibm=self.mathlib
        elif self.try_link("#define FPU_UNICOS\n" + testcode,
                           libraries=[self.mathlib],
                           include_dirs=self.unix_include_dirs):
            print "using FPU_UNICOS (SIGFPE delivery)"
            self.configfile.write('#define FPU_UNICOS\n')
            fpedef="-DFPU_UNICOS"
            fpelibm=self.mathlib

        if os.environ.has_key('fpedef'):
            if self.try_link("#define FPU_IGNORE\n" + testcode,
                             libraries=[self.mathlib],
                             include_dirs=self.unix_include_dirs):
                print "using FPU_IGNORE (SIGFPE delivery)"
                configfile.write('#define FPU_IGNORE\n')
                fpedef="-DFPU_IGNORE"
            else:
                print "FATAL unable to build SIGFPE fputest? (fputest.c, fpuset.c)"
                self.fatality=1

        if fpelib:
            self.configfile.write("FPELIB="+fpelib+"\n")
        else:
            self.configfile.write("FPELIB=\n")

        if fpelibm:
            self.configfile.write("FPELIBM="+fpelibm+"\n")
        else:
            self.configfile.write("FPELIBM=\n")

        if fpedef:
            # on IRIX be sure that TRAP_FPE environment variable is turned off
            if os.environ.has_key('TRAP_FPE'): del os.environ["TRAP_FPE"]
            testcode = "#define " + fpedef[2:] + "\n" + testcode
            libraries = []
            if fpelib: libraries.append(fpelib)
            if fpelibm: libraries.append(fpelibm)
            if not self.try_run(testcode,
                                libraries=libraries,
                                include_dirs=self.unix_include_dirs):
                print
                print "*************************WARNING***************************"
                print "*** play/unix configuration failed to get SIGFPE delivered"
                print "*** read the notes in play/unix/README.fpu"
                print "*************************WARNING***************************"
                print

    def find_user_name(self,configfile):
        # find function to get user name
        testcode = """\
/* check settings of: NO_PASSWD */
#define CONFIG_SCRIPT
#include "usernm.c"
int
main(int argc, char *argv[])
{
  int value;
  char *u = p_getuser(); \
  value = (u!=0);
  return value;
}
"""
        if self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "using POSIX getlogin(), getpwuid(), getuid() functions"
        elif self.try_link("#define USE_PASSWD\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "fallback to cuserid(), POSIX getlogin() family missing"
            configfile.write('#define NO_PASSWD\n')
        else:
            print "FATAL cuserid(), POSIX getlogin() family both missing (usernm.c)"
            self.fatality = 1

    def find_tiocgpgrp(self, configfile):
        # find function to get controlling terminal process group
        testcode = """\
/* check settings of: USE_TIOCGPGRP_IOCTL */
#define CONFIG_SCRIPT
#include "uinbg.c"
int
main(int argc, char *argv[])
{
  int value;
  value = u_in_background();
  return value;
}
"""
        if self.try_compile(testcode,include_dirs=self.unix_include_dirs):
            pass
        elif self.try_compile("#define USE_POSIX_GETPGRP\n"+testcode,
                              include_dirs=self.unix_include_dirs):
            print "using strict POSIX getpgrp prototype"
            testcode = "#define USE_POSIX_GETPGRP\n" + testcode
            configfile.write('#define USE_POSIX_GETPGRP\n')
        if self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "using POSIX tcgetpgrp() function"
        elif self.try_link('#define USE_TIOCGPGRP_IOCTL=<sys/termios.h>\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "fallback to TIOCGPGRP in sys/termios.h, POSIX tcgetpgrp() missing"
            configfile.write('#define USE_TIOCGPGRP_IOCTL <sys/termios.h>\n')
        elif self.try_link('#define USE_TIOCGPGRP_IOCTL=<sgtty.h>\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "fallback to TIOCGPGRP in sgtty.h, POSIX tcgetpgrp() missing"
            configfile.write('#define USE_TIOCGPGRP_IOCTL <sgtty.h>\n')
        else:
            print "FATAL unable to find TIOCGPGRP ioctl header (uinbg.c)"
            print "  (you can patch config.0h by hand if you know header)"
            configfile.write('#define USE_TIOCGPGRP_IOCTL <???>\n')
            self.fatality = 1

    def find_cwd(self, configfile):
        # find function to get current working directory
        testcode = """\
/* check settings of: USE_GETWD */
#define CONFIG_SCRIPT
#include <unistd.h>
static char dirbuf[1024];
#ifdef USE_GETWD
#define getcwd(x,y) getwd(x)
#endif
int
main(int argc, char *argv[])
{
  int value;
  char *u = getcwd(dirbuf, 1024);
  value = (u!=0);
  return value;
}
"""
        if self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "using POSIX getcwd() function"
        elif self.try_link("#define USE_GETWD\n"+testcode,
                           include_dirs=self.unix_include_dirs):
            print "fallback to getwd(), POSIX getcwd() missing"
            configfile.write('#define USE_GETWD\n')
        else:
            print "FATAL getcwd(), getwd() both missing (dir.c)"
            self.fatality=1

    def find_dirent(self, configfile):
        # find headers required to read directories
        testcode = """\
/* check settings of: DIRENT_HEADER USE_GETWD */
#define TEST_DIRENT
#define CONFIG_SCRIPT
#include "dir.c"
p_twkspc p_wkspc;
int
main(int argc, char *argv[])
{
  int value;
  p_dir *d = p_dopen("no/such/thing");
  char *l = p_dnext(d, &value);
  value = p_chdir(l) || p_rmdir(l) || p_mkdir(l);
  return value;
}
"""
        if self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "using POSIX dirent.h header for directory ops"
        elif self.try_link('#define DIRENT_HEADER=<sys/dir.h>\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using sys/dir.h header for directory ops"
            configfile.write('#define DIRENT_HEADER <sys/dir.h>\n')
        elif self.try_link('#define DIRENT_HEADER=<sys/ndir.h>\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using sys/ndir.h header for directory ops"
            configfile.write('#define DIRENT_HEADER <sys/ndir.h>')
        elif self.try_link('#define DIRENT_HEADER=<ndir.h>\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using ndir.h header for directory ops"
            configfile.write('#define DIRENT_HEADER <ndir.h>')
        else:
            print "FATAL dirent.h, sys/dir.h, sys/ndir.h, ndir.h all missing (dir.c)"
            self.fatality=1

    def find_poll(self,configfile):
        # find headers and functions required for poll/select functionality
        testcode = """\
/* check settings of: USE_SYS_POLL_H USE_SELECT HAVE_SYS_SELECT_H
                      NO_SYS_TIME_H NEED_SELECT_PROTO */
#define CONFIG_SCRIPT
#define TEST_POLL
#include "uevent.c"
int
main(int argc, char *argv[])
{
  int p = u_poll(1000); \
  return 0;
}
"""
        if self.try_link(testcode,include_dirs=self.unix_include_dirs):
            print "using poll(), poll.h header"
        elif self.try_link('#define USE_SYS_POLL_H\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using poll(), sys/poll.h header"
            configfile.write('#define USE_SYS_POLL_H\n')
        elif self.try_link('#define USE_SELECT\n'+'#define HAVE_SYS_SELECT_H\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using select(), sys/select.h header"
            configfile.write('#define USE_SELECT\n')
            configfile.write('#define HAVE_SYS_SELECT_H\n')
        elif self.try_link('#define USE_SELECT\n'+'#define NEED_SELECT_PROTO\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using select(), sys/time.h, sys/types.h headers"
            configfile.write('#define USE_SELECT\n')
            configfile.write('#define NEED_SELECT_PROTO\n')
        elif self.try_link('#define USE_SELECT\n'+'#define NO_SYS_TIME_H\n'+'#define NEED_SELECT_PROTO\n'+testcode,
                           include_dirs=self.unix_include_dirs):
            print "using select(), time.h, sys/types.h headers"
            configfile.write('#define USE_SELECT\n')
            configfile.write('#define NO_SYS_TIME_H\n')
            configfile.write('#define NEED_SELECT_PROTO\n')
        elif  self.try_link('#define USE_SELECT\n'+testcode,
                            include_dirs=self.unix_include_dirs):
            print "using select(), sys/time.h header"
            configfile.write('#define USE_SELECT')
        elif  self.try_link('#define USE_SELECT\n'+'#define NO_SYS_TIME_H\n'+testcode,
                            include_dirs=self.unix_include_dirs):
            print "using select(), time.h header"
            configfile.write('#define USE_SELECT\n')
            configfile.write('#define NO_SYS_TIME_H\n')
        else:
            print "FATAL neither poll() nor select() usable? (uevent.c, upoll.c)"
            self.fatality=1
#----------------------------------------------------------------------
    def config_x11(self):
        if sys.platform in ['win32','cygwin']:
            return
        if os.environ.has_key('NO_XLIB'):
            return
        print
        print "  ============= begin play/x11 configuration =============="
        print
        from string import replace
        self.fatality=0

        x11_info = get_info('x11')
        if x11_info:
            print "found X Window System, X11 headers and libraries"
            xinc = x11_info.get('include_dirs','')
            xinc = ' '.join(['-I'+d for d in xinc])
            xlib = x11_info.get('library_dirs','')
            xlib = ' '.join(['-L'+d for d in xlib])
	    if xinc:
            	print "  - using X11 header switch "+xinc
            print "  - using X11 loader switch "+xlib
            self.configfile.write("XINC="+xinc+"\n")
            self.configfile.write("XLIB="+xlib+"\n")
            print "appended to Make.cfg"
        else:
            print "FATAL unable to find X11 includes (play/x11) $xinc"
            self.fatality=1

        print
        print "  ============== end play/x11 configuration ==============="
        return
        # figure out directories to compile and load with X11
        X11BASE=os.environ.get('X11BASE','/no/suggested/x11dir')

        # directory list is from autoconf, except openwin promoted near top
        xlist = [X11BASE+"/include",
                 "/usr/X11R6/include",
                 "/usr/X11R5/include",
                 "/usr/X11R4/include",
                 "/usr/include/X11R6",
                 "/usr/include/X11R5",
                 "/usr/include/X11R4",
                 "/usr/openwin/include",
                 "/usr/openwin/share/include",
                 "/usr/local/X11R6/include",
                 "/usr/local/X11R5/include",
                 "/usr/local/X11R4/include",
                 "/usr/local/include/X11R6",
                 "/usr/local/include/X11R5",
                 "/usr/local/include/X11R4",
                 "/usr/X11/include",
                 "/usr/include/X11",
                 "/usr/local/X11/include"
                 "/usr/local/include/X11",
                 "/usr/X386/include",
                 "/usr/x386/include",
                 "/usr/XFree86/include/X11",
                 "/usr/include",
                 "/usr/local/include",
                 "/usr/unsupported/include",
                 "/usr/athena/include",
                 "/usr/local/x11r5/include",
                 "/usr/lpp/Xamples/include"]
        testcode = """\
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>
extern XVisualInfo xvi;
XVisualInfo xvi;             /* XVisualInfo declared in Xutil.h */
int main(int argc, char *argv[])
{
  Display *dpy = XOpenDisplay("nosuchserver:0.0");
  xvi.screen = XK_BackSpace;  /* XK_BackSpace defined in keysym.h */
  xvi.red_mask = XA_PRIMARY;  /* XA_PRIMARY defined in Xatom.h */
  xvi.depth = XC_crosshair;   /* XC_crosshair defined in cursorfont.h */
  XCloseDisplay(dpy);
  return 0;
}
"""
        xinc = ""
        xlib = ""
        xfound = 0
        if self.try_compile(testcode):
            xfound=1
        else:
            for d in xlist:
                if os.path.exists(d+'/X11/Xlib.h'):
                    if self.try_compile(testcode,include_dirs=[d]):
                        xfound = 1
                        xinc = d
                        xlib = replace(d,"include","lib")
                        break
        if not xfound:
            print "FATAL unable to find X11 includes (play/x11) $xinc"
            self.fatality=1
        else:
            if self.try_link(testcode,include_dirs=[xinc],libraries=['X11']):
                xlib=""
                xfound=2
            elif xlib and self.try_link(testcode,include_dirs=[xinc],library_dirs=[xlib],libraries=['X11']):
                xfound=2
            else:
                xlist = [replace(directory,'include','lib') for directory in xlist]
		if xinc:
                    for d in xlist:
                        if self.try_link(testcode,include_dirs=[xinc],library_dirs=[d],libraries=['X11']):
                            xlib = d
                            xfound=2
                            break
		else:
                    for d in xlist:
                        if self.try_link(testcode,library_dirs=[d],libraries=['X11']):
                            xlib = d
                            xfound=2
                            break
        if xfound:
            print "found X Window System, X11 headers and libraries"
	    if xinc:
            	print "  - using X11 header switch -I"+xinc
	    else:
            	print "  - using X11 header switch [none]"
            print "  - using X11 loader switch -L"+xlib
        else:
            print "FATAL unable to find X11 libraries (play/x11) $xlib"
            self.fatality=1
        if xinc:
	    self.configfile.write("XINC=-I"+xinc+"\n")
	else:
	    self.configfile.write("XINC=\n")
        self.configfile.write("XLIB=-L"+xlib+"\n")

        print "appended to Make.cfg"
        print
        print "  ============== end play/x11 configuration ==============="



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
from GistPlotter import *
GraphicsError = "GraphicsError"
import os
try:
   graphics = os.environ["PYGRAPH"]
except KeyError:
   graphics = "Gist"
 
if graphics [0:3] == "Nar" :
   import NarPlotter
elif graphics == "Gist" :
   import GistPlotter
else :
   raise GraphicsError , \
      graphics + " is an unknown graphics package. Check PYGRAPH " + \
         "environment variable."

def span (lb, ub, n) :
   if n < 2: raise ValueError, "Third argument must be at least 2."
   b = lb
   a = (ub - lb) / (n - 1.0)
   return map(lambda x, A=a, B=b: A*x+B,range(n))

def a3 (lb, ub, n) :
   return reshape (array(n*span(lb,ub,n), Float), (n,n))

def mag ( *args ) :
   r = 0
   for i in range (len (args)) :
      r = r + args[i] * args[i]
   return sqrt (r)

def a2 (lb, ub, n) :
   return reshape (array ((n - 1) * spanz (lb, ub, n),  Float), (n-1, n-1))

def spanz (lb, ub, n) :
   if n < 3 : raise ValueError, "3rd argument must be at least 3"
   c = 0.5 * (ub - lb) / (n - 1.0)
   b = lb + c
   a = (ub -c -b) / (n - 2.0)
   return map (lambda x, A = a, B = b: A * x + B, range (n - 1))

def init ( self ) :
    self.t = 2*pi*arange (400, typecode = Float) / 399.0
    self.na1 = 1
    self.nb1 = 5
    self.na2 = 2
    self.nb2 = 7
    self.rc1 = 40.
    self.rc2 = 160.
    self.size = 40.
    self.phase = self.theta = 0.
    self.dtheta = pi / (self.nsteps - 1)
    self.dphase = 2 * pi / (self.nsteps - 1)

def calc1 ( self ) :
    self.cost = cos (self.theta)
    self.sint = sin (self.theta)
    self.x = self.rc1 * self.cost + self.size * cos (self.na1 * self.t)
    self.y = self.rc1 * self.sint + self.size * sin (self.nb1 * self.t +
       self.phase)

def calc2 ( self ) :
    self.x = self.rc2 * self.cost + self.size * cos (self.na2 * self.t)
    self.y = self.rc2 * self.sint + self.size * sin (self.nb2 * self.t +
       self.phase)

def incr  ( self ) :
    self.theta = self.theta + self.dtheta
    self.phase = self.phase + self.dphase

from curve import Curve
from graph2d import *
def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return
 
print "\nComprehensive graphics test (sort of) for Python interface"
print "with Narcisse and (mainly) Gist. This mirrors Lee's low level"
print "routine. Each frame will be described at the terminal; compare"
print "what you see with the description, then hit <RETURN> to see"
print "the next test, or ^C (ctrl-C) to escape.\n"
print "Type 'demo()' to begin.\n"

from quadmesh import *
from cellarray import *
from lines import *
from animation2d import *


def demo ( ) :
   c1 = Curve ( y = [0,1] , marks = 1 , marker = "A")
   pl = Plotter ( dpi = 75 )
   g1 = Graph2d ( c1, plotter = pl , titles = "Curve marked with A" ,
               title_colors = "blue")
   g1.plot ( )
   print "\nA small (75 dpi) window with line marked A from (0,0) to (1,1)."
   paws ( )
#  pl = Plotter ( dpi = 100 )
#  g1 = Graph2d ( c1, plotter = pl , titles = "Curve marked with A" ,
#              title_colors = "fg")
#  g1.plot ( )
#  print "\nA large (100 dpi) window with line marked A from (0,0) to (1,1)."
#  paws ( )
   c2 = Curve ( y = [1,0] , marks = 1 , marker = "B")
   g1.add (c2)
   g1.change (titles = "New curve marked B.")
   g1.plot ( )
   print "\nAdded line marked B from (0,1) to (1,0) in previous plot."
   paws ( )
   c1.set ( x = [1,2] )
   c2.set ( x = [1,2] )
   g1.change (axis_scales = "loglin", titles = "Same, x axis now logarithmic.")
   g1.plot ( )
   print "\nSame, x axis now logarithmic."
   paws ( )
   g1.change (x_axis_scale = "lin", titles = "x axis back to linear.")
   g1.plot ( )
   print "\nSame, x axis now linear again."
   paws ( )
   g1.change(axis_limits=[[1.2,1.8],[0.2,0.8]],
          titles="Limits now 1.2<x<1.8, 0.2<y<0.8.")
   g1.plot ( )
   print "\nLimits now 1.2<x<1.8, 0.2<y<0.8."
   paws ( )
#  g1.change(y_axis_limits=[0.4,0.6],
#         titles="Limits now 1.2<x<1.8, 0.4<y<0.6")
#  g1.plot ( )
#  print "\nLimits now 1.2<x<1.8, 0.4<y<0.6"
#  paws ( )
   g1.change(axis_limits="defaults",
          titles="Limits now back to extreme values.")
   g1.plot ( )
   print "\nLimits now back to extreme values."
   paws ( )
   x=10*pi*arange(200, typecode = Float)/199.0
   c1 = Curve ( x = x , y = sin(x),marks = 1, marker= "A")
   g1.delete (2)
   g1.delete (1)
   g1.add (c1)
   g1.change (titles = "Five cycles of a sine wave, marked A.")
   g1.plot ( )
   print "\nFive cycles of a sine wave."
   paws ( )
   c1.set(marks = 0, width = 6 , type = "dash")
   g1.change (titles = "A mark gone, bold dashed.")
   g1.plot ( )
   print "\nTurn off A marker, plot curve as bold dashed."
   paws ( )
   x=2*pi*arange(200, typecode = Float)/199.0
   crvs = []
   for i in range (1,7) :
      r = 0.5*i -(5-0.5*i)*cos(x)
      s = `i`
      crvs.append(Curve(y=r*sin(x),x=r*cos(x),marks=0, color=-4-i,label=s))
   g1=Graph2d(crvs,plotter = pl,titles="Nested cardioids in colors")
   g1.plot ( )
   print "\nNested cardioids in primary and secondary colors."
   paws()
   g1.change(titles = ["colors","cardioids","Nested","in many"],
          title_colors = ["red","yellow","magenta","blue"])
   g1.plot ( )
   print "\nThe same with four titles in many colors."
   paws()
   mks = ['.','+','*','o','x','A']
   for i in range (1,7) :
      crvs[i-1].set (color="fg", type = 0, marker = mks[i-1])
   g1.change (titles = "Marked curves", title_colors = "fg")
   g1.plot ( )
   print "\nChanges color to foreground, types to no lines."
   print "Markers are point, plus, asterisk, circle, cross, and A."
   paws()
   for i in range (1,6) :
      crvs[i-1].set (marks = 0, type = i-1)
   crvs[i].set (type = 1, width = 4)
   g1.change (titles = "Different curve types.")
   print "\nChanges line types to solid, dash, dot, dashdot, dashdotdot."
   print "Outermost cardioid becomes a thick solid curve."
   g1.plot()
   paws ()
   ValueError = "ValueError"
   x=a3(-1,1,26)
   y=transpose (x)
   zz=x+1j*y
   zz=5.*zz/(5.+zz*zz)
   xx=zz.real
   yy=zz.imaginary
   q1 = QuadMesh ( x=array(xx, copy = 1), y=array(yy, copy = 1) )
   g1.new(q1, plotter = pl, titles = "Round mesh with bites out of sides",
      axis_limits = [[min(ravel(xx)),max(ravel(xx))],
                     [min(ravel(yy)),max(ravel(yy))]])
   g1.plot()
   print "\nQuadrilateral mesh -- round with bites out of its sides."
   paws (  )
   q1.set (boundary = 1, vx = x+.5, vy=y-.5,ktype = "none", ltype = "none")
   g1.change(titles = "Vector field on mesh.")
   g1.plot ( )
   print "\nVelocity vectors. Try zooming and panning with mouse."
   paws ( )
   q1.set (vx = None, vy = None, z = mag (x+.5,y-.5),marks = 1,
      boundary=1,boundary_type="dash",marker="A")
   g1.change(titles = "Lettered contour plot, dashed boundary.")
   g1.plot ( )
   print "\nLettered contours, with boundary dashed."
   paws ( )
   q1.set (filled=1, type = "dash", color = "white")
   g1.plot ( )
   print "\nFilled lettered dashed contours, with boundary dashed."
   paws ()
   q1.set(marks = 0,boundary_type="solid",levels=array([0.5,1.0,1.5]),
       filled=1, contours = 1, type = "dash", color = "white",
       width = 3)
   g1.change(titles = "Filled mesh, dashed contours")
   g1.plot ( )
   print "\nFilled mesh (color only) with three dashed contours overlaid."
   paws()
   print "\nAfter each new palette is installed, hit <RETURN> for the next."
   pal=["heat.gp","stern.gp","rainbow.gp","gray.gp","yarg.gp","earth.gp"]
   for i in range(6):
      g1.quick_plot(color_card = pal[i],titles = "Palette " + pal[i])
   #  g1.plot ( )
      paws ( )
   pal = ["vg.gs", "boxed.gs", "vgbox.gs", "nobox.gs", "work.gs"]
   q1.set (marks = 1,boundary_type="dash",levels = 8, filled = 0, color = "fg")
   for i in range (5) :
      pl.set_style (pal [i])
##    g1 = Graph2d (q1, plotter = pl, titles = "Style Name: " + pal[i])
      g1.change (plotter = pl, titles = "Style Name: " + pal[i])
      g1.plot ( )
      paws ( )
   x=a3(-6,6,200)
   y=transpose (x)
   r=mag(y,x)
   theta=arctan2(y,x)
   funky=cos(r)**2*cos(3*theta)
   c1 = CellArray(z=funky)
   g1.replace (1,c1)
   g1.change (color_card = "earth.gp",
           titles ="Cell array, three cycles in theta,r",
           axis_limits = "defaults")
   g1.plot()
   paws()

   theta = a2 (0, 2*pi, 18)
   x = cos (theta)
   y = sin (theta)

   ln = Lines (x0 = x, y0 = y, x1 = transpose (x), y1 = transpose (y))
   g1 = Graph2d (ln, plotter = pl, xyequal = 1, titles = "Seventeen pointed star.")
   g1.plot ()
   print "\nAll 17 pointed stars."
   paws ( )

   print "\nNext we will see a lissajous-style animation."
   print "First run without animation mode."
   print "Second run with animation mode."


   anim = Animation2d ( initialize = init, calculations = [calc1, calc2],
                     update = incr, animation = 0, nsteps = 200 )

   g1 = Graph2d ( anim , plotter = pl)
   g1.plot ( )
   paws ( )
   anim.set (animation = 1)
   g1.plot ( )




#!/usr/bin/env python

import os
from glob import glob
from scipy_distutils.core import Extension
from scipy_distutils.misc_util import get_path, default_config_dict, dot_join
from scipy_distutils import system_info


def configuration(parent_package='',parent_path=None):
    package = 'optimize'
    config = default_config_dict(package, parent_package)
    local_path = get_path(__name__,parent_path)    

    minpack = ('minpack',{'sources':
                          glob(os.path.join(local_path,'minpack','*.f'))})

    sources = ['_minpackmodule.c']
    sources = [os.path.join(local_path,x) for x in sources]
    ext = Extension(dot_join(parent_package, package, '_minpack'),
                    sources, libraries = [minpack])
    config['ext_modules'].append(ext)

    rootfind = glob(os.path.join(local_path,'Zeros','*.c'))
    roothead = os.path.join(local_path,'zeros.h')
    config['libraries'].append(('rootfind',{'sources':rootfind,
                                            'headers':roothead}))
    sources = ['zeros.c']
    sources = [os.path.join(local_path,x) for x in sources]    
    ext = Extension(dot_join(parent_package, package, '_zeros'),
                    sources, libraries=['rootfind'])
    config['ext_modules'].append(ext)

    lapack = system_info.get_info('lapack_opt')
    sources = ['lbfgsb.pyf','routines.f']
    sources = [os.path.join(local_path,'lbfgsb-0.9',x) for x in sources]
    ext = Extension(dot_join(parent_package, package, "_lbfgsb"),
		    sources=sources, **lapack)
    config['ext_modules'].append(ext)

    sources = ['moduleTNC.c', 'tnc.c']
    sources = [os.path.join(local_path,'tnc',x) for x in sources]
    ext = Extension(dot_join(parent_package, package, 'moduleTNC'), 
		    sources=sources)
    config['ext_modules'].append(ext)

    sources = ['cobyla.pyf','cobyla2.f','trstlp.f']
    sources = [os.path.join(local_path,'cobyla',x) for x in sources]
    ext = Extension(dot_join(parent_package, package, '_cobyla'), sources=sources)
    config['ext_modules'].append(ext)

    sources = ['minpack2.pyf', 'dcsrch.f', 'dcstep.f']
    sources = [os.path.join(local_path,'minpack2',x) for x in sources]    
    ext = Extension(dot_join(parent_package, package, 'minpack2'), sources=sources)
    config['ext_modules'].append(ext)
    
    return config

if __name__ == '__main__':    
    from scipy_distutils.core import setup
    setup(**configuration(parent_path=''))



""" Test refcounting and behavior of SCXX.
"""
import unittest
import time
import os,sys

from scipy_test.testing import *
set_package_path()
from weave import inline_tools
restore_path()

class test_object_construct(unittest.TestCase):
    #------------------------------------------------------------------------
    # Check that construction from basic types is allowed and have correct
    # reference counts
    #------------------------------------------------------------------------
    def check_int(self,level=5):
        # strange int value used to try and make sure refcount is 2.
        code = """
               py::object val = 1001;
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == 1001
    def check_float(self,level=5):
        code = """
               py::object val = (float)1.0;
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == 1.0
    def check_double(self,level=5):
        code = """
               py::object val = 1.0;
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == 1.0
    def check_complex(self,level=5):
        code = """
               std::complex<double> num = std::complex<double>(1.0,1.0);
               py::object val = num;
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == 1.0+1.0j
    def check_string(self,level=5):
        code = """
               py::object val = "hello";
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == "hello"

    def check_std_string(self,level=5):
        code = """
               std::string s = std::string("hello");
               py::object val = s;
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == "hello"
            
class test_object_print(unittest.TestCase):
    #------------------------------------------------------------------------
    # Check the object print protocol.  
    #------------------------------------------------------------------------
    def check_stdout(self,level=5):
        code = """
               py::object val = "how now brown cow";
               val.print(stdout);
               """
        res = inline_tools.inline(code)
        # visual check on this one.
    def check_stringio(self,level=5):
        import cStringIO
        file_imposter = cStringIO.StringIO()
        code = """
               py::object val = "how now brown cow";
               val.print(file_imposter);
               """
        res = inline_tools.inline(code,['file_imposter'])
        print file_imposter.getvalue()
        assert file_imposter.getvalue() == "'how now brown cow'"

    def check_failure(self,level=5):
        code = """
               FILE* file = 0;
               py::object val = "how now brown cow";
               val.print(file);
               """
        try:       
            res = inline_tools.inline(code)
        except:
            # error was supposed to occur.
            pass    

                    
class test_object_cast(unittest.TestCase):
    def check_int_cast(self,level=5):
        code = """
               py::object val = 1;
               int raw_val = val;
               """
        inline_tools.inline(code)
    def check_double_cast(self,level=5):
        code = """
               py::object val = 1.0;
               double raw_val = val;
               """
        inline_tools.inline(code)
    def check_float_cast(self,level=5):
        code = """
               py::object val = 1.0;
               float raw_val = val;
               """
        inline_tools.inline(code)
    def check_complex_cast(self,level=5):
        code = """
               std::complex<double> num = std::complex<double>(1.0,1.0);
               py::object val = num;
               std::complex<double> raw_val = val;
               """
        inline_tools.inline(code)
    def check_string_cast(self,level=5):
        code = """
               py::object val = "hello";
               std::string raw_val = val;
               """
        inline_tools.inline(code)
                    
# test class used for testing python class access from C++.
class foo:
    def bar(self):
        return "bar results"
    def bar2(self,val1,val2):
        return val1, val2
    def bar3(self,val1,val2,val3=1):
        return val1, val2, val3

class str_obj:
            def __str__(self):
                return "b"

class test_object_hasattr(unittest.TestCase):
    def check_string(self,level=5):
        a = foo()
        a.b = 12345
        code = """
               return_val = a.hasattr("b");               
               """
        res = inline_tools.inline(code,['a'])
        assert res
    def check_std_string(self,level=5):
        a = foo()
        a.b = 12345
        attr_name = "b"
        code = """
               return_val = a.hasattr(attr_name);               
               """
        res = inline_tools.inline(code,['a','attr_name'])
        assert res        
    def check_string_fail(self,level=5):
        a = foo()
        a.b = 12345
        code = """
               return_val = a.hasattr("c");               
               """
        res = inline_tools.inline(code,['a'])
        assert not res
    def check_inline(self,level=5):
        """ THIS NEEDS TO MOVE TO THE INLINE TEST SUITE
        """
        a = foo()
        a.b = 12345
        code = """
               throw_error(PyExc_AttributeError,"bummer");               
               """
        try:
            before = sys.getrefcount(a)
            res = inline_tools.inline(code,['a'])
        except AttributeError:
            after = sys.getrefcount(a)
            try: 
                res = inline_tools.inline(code,['a'])
            except:
                after2 = sys.getrefcount(a)
            print "after and after2 should be equal in the following"        
            print 'before, after, after2:', before, after, after2
            pass    

    def check_func(self,level=5):
        a = foo()
        a.b = 12345
        code = """
               return_val = a.hasattr("bar");               
               """
        res = inline_tools.inline(code,['a'])
        assert res

class test_object_attr(unittest.TestCase):

    def generic_attr(self,code,args=['a']):
        a = foo()
        a.b = 12345
                
        before = sys.getrefcount(a.b)
        res = inline_tools.inline(code,args)
        assert res == a.b
        del res
        after = sys.getrefcount(a.b)
        assert after == before

    def check_char(self,level=5):
        self.generic_attr('return_val = a.attr("b");')

    def check_char_fail(self,level=5):
        try:
            self.generic_attr('return_val = a.attr("c");')
        except AttributeError:
            pass
            
    def check_string(self,level=5):
        self.generic_attr('return_val = a.attr(std::string("b"));')

    def check_string_fail(self,level=5):
        try:
            self.generic_attr('return_val = a.attr(std::string("c"));')
        except AttributeError:
            pass    

    def check_obj(self,level=5):
        code = """
               py::object name = "b";
               return_val = a.attr(name);
               """ 
        self.generic_attr(code,['a'])

    def check_obj_fail(self,level=5):
        try:
            code = """
                   py::object name = "c";
                   return_val = a.attr(name);
                   """ 
            self.generic_attr(code,['a'])
        except AttributeError:
            pass    
            
    def check_attr_call(self,level=5):
        a = foo()
        res = inline_tools.inline('return_val = a.attr("bar").call();',['a'])
        first = sys.getrefcount(res)
        del res
        res = inline_tools.inline('return_val = a.attr("bar").call();',['a'])
        second = sys.getrefcount(res)
        assert res == "bar results"
        assert first == second

class test_object_set_attr(unittest.TestCase):

    def generic_existing(self, code, desired):
        args = ['a']
        a = foo()
        a.b = 12345                
        res = inline_tools.inline(code,args)
        assert a.b == desired

    def generic_new(self, code, desired):
        args = ['a']
        a = foo()
        res = inline_tools.inline(code,args)
        assert a.b == desired

    def check_existing_char(self,level=5):
        self.generic_existing('a.set_attr("b","hello");',"hello")
    def check_new_char(self,level=5):
        self.generic_new('a.set_attr("b","hello");',"hello")
    def check_existing_string(self,level=5):
        self.generic_existing('a.set_attr("b",std::string("hello"));',"hello")
    def check_new_string(self,level=5):
        self.generic_new('a.set_attr("b",std::string("hello"));',"hello")
    def check_existing_object(self,level=5):
        code = """
               py::object obj = "hello";
               a.set_attr("b",obj);
               """
        self.generic_existing(code,"hello")
    def check_new_object(self,level=5):
        code = """
               py::object obj = "hello";
               a.set_attr("b",obj);
               """
        self.generic_new(code,"hello")
    def check_new_fail(self,level=5):
        try:
            code = """
                   py::object obj = 1;
                   a.set_attr(obj,"hello");
                   """
            self.generic_new(code,"hello")
        except:
            pass    

    def check_existing_int(self,level=5):
        self.generic_existing('a.set_attr("b",1);',1)
    def check_existing_double(self,level=5):
        self.generic_existing('a.set_attr("b",1.0);',1.0)
    def check_existing_complex(self,level=5):
        code = """
               std::complex<double> obj = std::complex<double>(1,1);
               a.set_attr("b",obj);
               """
        self.generic_existing(code,1+1j)
    def check_existing_char1(self,level=5):
        self.generic_existing('a.set_attr("b","hello");',"hello")        
    def check_existing_string1(self,level=5):
        code = """
               std::string obj = std::string("hello");
               a.set_attr("b",obj);
               """
        self.generic_existing(code,"hello")

class test_object_del(unittest.TestCase):
    def generic(self, code):
        args = ['a']
        a = foo()
        a.b = 12345                
        res = inline_tools.inline(code,args)
        assert not hasattr(a,"b")

    def check_char(self,level=5):
        self.generic('a.del("b");')
    def check_string(self,level=5):
        code = """
               std::string name = std::string("b");
               a.del(name);
               """
        self.generic(code)
    def check_object(self,level=5):
        code = """
               py::object name = py::object("b");
               a.del(name);
               """
        self.generic(code)

class test_object_cmp(unittest.TestCase):
    def check_equal(self,level=5):
        a,b = 1,1
        res = inline_tools.inline('return_val = (a == b);',['a','b'])
        assert res == (a == b)
    def check_equal_objects(self,level=5):
        class foo:
            def __init__(self,x):
                self.x = x
            def __cmp__(self,other):
                return cmp(self.x,other.x)
        a,b = foo(1),foo(2)
        res = inline_tools.inline('return_val = (a == b);',['a','b'])
        assert res == (a == b)
    def check_lt(self,level=5):
        a,b = 1,2
        res = inline_tools.inline('return_val = (a < b);',['a','b'])
        assert res == (a < b)
    def check_gt(self,level=5):
        a,b = 1,2
        res = inline_tools.inline('return_val = (a > b);',['a','b'])
        assert res == (a > b)
    def check_gte(self,level=5):
        a,b = 1,2
        res = inline_tools.inline('return_val = (a >= b);',['a','b'])
        assert res == (a >= b)
    def check_lte(self,level=5):
        a,b = 1,2
        res = inline_tools.inline('return_val = (a <= b);',['a','b'])
        assert res == (a <= b)
    def check_not_equal(self,level=5):
        a,b = 1,2
        res = inline_tools.inline('return_val = (a != b);',['a','b'])
        assert res == (a != b)
    def check_int(self,level=5):
        a = 1
        res = inline_tools.inline('return_val = (a == 1);',['a'])
        assert res == (a == 1)
    def check_int2(self,level=5):
        a = 1
        res = inline_tools.inline('return_val = (1 == a);',['a'])
        assert res == (a == 1)
    def check_unsigned_long(self,level=5):
        a = 1
        res = inline_tools.inline('return_val = (a == (unsigned long)1);',['a'])
        assert res == (a == 1)
    def check_double(self,level=5):
        a = 1
        res = inline_tools.inline('return_val = (a == 1.0);',['a'])
        assert res == (a == 1.0)
    def check_char(self,level=5):
        a = "hello"
        res = inline_tools.inline('return_val = (a == "hello");',['a'])
        assert res == (a == "hello")
    def check_std_string(self,level=5):
        a = "hello"
        code = """
               std::string hello = std::string("hello");
               return_val = (a == hello);
               """
        res = inline_tools.inline(code,['a'])
        assert res == (a == "hello")

class test_object_repr(unittest.TestCase):
    def check_repr(self,level=5):
        class foo:
            def __str__(self):
                return "str return"
            def __repr__(self):
                return "repr return"
        a = foo()                
        res = inline_tools.inline('return_val = a.repr();',['a'])
        first = sys.getrefcount(res)
        del res
        res = inline_tools.inline('return_val = a.repr();',['a'])
        second = sys.getrefcount(res)
        assert first == second
        assert res == "repr return"

class test_object_str(unittest.TestCase):
    def check_str(self,level=5):
        class foo:
            def __str__(self):
                return "str return"
            def __repr__(self):
                return "repr return"
        a = foo()                
        res = inline_tools.inline('return_val = a.str();',['a'])
        first = sys.getrefcount(res)
        del res
        res = inline_tools.inline('return_val = a.str();',['a'])
        second = sys.getrefcount(res)
        assert first == second
        print res
        assert res == "str return"

class test_object_unicode(unittest.TestCase):
    # This ain't going to win awards for test of the year...
    def check_unicode(self,level=5):
        class foo:
            def __repr__(self):
                return "repr return"
            def __str__(self):
                return "unicode"
        a= foo()                
        res = inline_tools.inline('return_val = a.unicode();',['a'])
        first = sys.getrefcount(res)
        del res
        res = inline_tools.inline('return_val = a.unicode();',['a'])
        second = sys.getrefcount(res)
        assert first == second
        assert res == "unicode"

class test_object_is_callable(unittest.TestCase):
    def check_true(self,level=5):
        class foo:
            def __call__(self):
                return 0
        a= foo()                
        res = inline_tools.inline('return_val = a.is_callable();',['a'])
        assert res
    def check_false(self,level=5):
        class foo:
            pass
        a= foo()                
        res = inline_tools.inline('return_val = a.is_callable();',['a'])
        assert not res

class test_object_call(unittest.TestCase):
    def check_noargs(self,level=5):
        def foo():
            return (1,2,3)
        res = inline_tools.inline('return_val = foo.call();',['foo'])
        assert res == (1,2,3)
        assert sys.getrefcount(res) == 2
    def check_args(self,level=5):
        def foo(val1,val2):
            return (val1,val2)
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               return_val = foo.call(args);
               """
        res = inline_tools.inline(code,['foo'])
        assert res == (1,"hello")
        assert sys.getrefcount(res) == 2
    def check_args_kw(self,level=5):
        def foo(val1,val2,val3=1):
            return (val1,val2,val3)
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               py::dict kw;
               kw["val3"] = 3;               
               return_val = foo.call(args,kw);
               """
        res = inline_tools.inline(code,['foo'])
        assert res == (1,"hello",3)
        assert sys.getrefcount(res) == 2
    def check_noargs_with_args(self,level=5):
        # calling a function that does take args with args 
        # should fail.
        def foo():
            return "blah"
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               return_val = foo.call(args);
               """
        try:
            first = sys.getrefcount(foo)
            res = inline_tools.inline(code,['foo'])
        except TypeError:
            second = sys.getrefcount(foo) 
            try:
                res = inline_tools.inline(code,['foo'])
            except TypeError:
                third = sys.getrefcount(foo)    
        # first should == second, but the weird refcount error        
        assert second == third
        
class test_object_mcall(unittest.TestCase):
    def check_noargs(self,level=5):
        a = foo()
        res = inline_tools.inline('return_val = a.mcall("bar");',['a'])
        assert res == "bar results"
        first = sys.getrefcount(res)
        del res
        res = inline_tools.inline('return_val = a.mcall("bar");',['a'])
        assert res == "bar results"
        second = sys.getrefcount(res)
        assert first == second
    def check_args(self,level=5):
        a = foo()
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               return_val = a.mcall("bar2",args);
               """
        res = inline_tools.inline(code,['a'])
        assert res == (1,"hello")
        assert sys.getrefcount(res) == 2
    def check_args_kw(self,level=5):
        a = foo()
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               py::dict kw;
               kw["val3"] = 3;
               return_val = a.mcall("bar3",args,kw);
               """
        res = inline_tools.inline(code,['a'])
        assert res == (1,"hello",3)
        assert sys.getrefcount(res) == 2
    def check_std_noargs(self,level=5):
        a = foo()
        method = "bar"
        res = inline_tools.inline('return_val = a.mcall(method);',['a','method'])
        assert res == "bar results"
        first = sys.getrefcount(res)
        del res
        res = inline_tools.inline('return_val = a.mcall(method);',['a','method'])
        assert res == "bar results"
        second = sys.getrefcount(res)
        assert first == second
    def check_std_args(self,level=5):
        a = foo()
        method = "bar2"
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               return_val = a.mcall(method,args);
               """
        res = inline_tools.inline(code,['a','method'])
        assert res == (1,"hello")
        assert sys.getrefcount(res) == 2
    def check_std_args_kw(self,level=5):
        a = foo()
        method = "bar3"
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               py::dict kw;
               kw["val3"] = 3;
               return_val = a.mcall(method,args,kw);
               """
        res = inline_tools.inline(code,['a','method'])
        assert res == (1,"hello",3)
        assert sys.getrefcount(res) == 2
    def check_noargs_with_args(self,level=5):
        # calling a function that does take args with args 
        # should fail.
        a = foo()
        code = """
               py::tuple args(2);
               args[0] = 1;
               args[1] = "hello";
               return_val = a.mcall("bar",args);
               """
        try:
            first = sys.getrefcount(a)
            res = inline_tools.inline(code,['a'])
        except TypeError:
            second = sys.getrefcount(a) 
            try:
                res = inline_tools.inline(code,['a'])
            except TypeError:
                third = sys.getrefcount(a)    
        # first should == second, but the weird refcount error        
        assert second == third

class test_object_hash(unittest.TestCase):
    def check_hash(self,level=5):
        class foo:
            def __hash__(self):
                return 123
        a= foo()                
        res = inline_tools.inline('return_val = a.hash(); ',['a'])
        print 'hash:', res
        assert res == 123

class test_object_is_true(unittest.TestCase):
    def check_true(self,level=5):
        class foo:
            pass
        a= foo()                
        res = inline_tools.inline('return_val = a.is_true();',['a'])
        assert res == 1
    def check_false(self,level=5):
        a= None                
        res = inline_tools.inline('return_val = a.is_true();',['a'])
        assert res == 0

class test_object_is_true(unittest.TestCase):
    def check_false(self,level=5):
        class foo:
            pass
        a= foo()                
        res = inline_tools.inline('return_val = a.not();',['a'])
        assert res == 0
    def check_true(self,level=5):
        a= None                
        res = inline_tools.inline('return_val = a.not();',['a'])
        assert res == 1    

class test_object_type(unittest.TestCase):
    def check_type(self,level=5):
        class foo:
            pass
        a= foo()                
        res = inline_tools.inline('return_val = a.type();',['a'])
        assert res == type(a)

class test_object_size(unittest.TestCase):
    def check_size(self,level=5):
        class foo:
            def __len__(self):
                return 10
        a= foo()                
        res = inline_tools.inline('return_val = a.size();',['a'])
        assert res == len(a)
    def check_len(self,level=5):
        class foo:
            def __len__(self):
                return 10
        a= foo()                
        res = inline_tools.inline('return_val = a.len();',['a'])
        assert res == len(a)
    def check_length(self,level=5):
        class foo:
            def __len__(self):
                return 10
        a= foo()                
        res = inline_tools.inline('return_val = a.length();',['a'])
        assert res == len(a)                            

from UserList import UserList
class test_object_set_item_op_index(unittest.TestCase):
    def check_list_refcount(self,level=5):
        a = UserList([1,2,3])            
        # temporary refcount fix until I understand why it incs by one.
        inline_tools.inline("a[1] = 1234;",['a'])        
        before1 = sys.getrefcount(a)
        after1 = sys.getrefcount(a)
        assert after1 == before1
    def check_set_int(self,level=5):
        a = UserList([1,2,3])            
        inline_tools.inline("a[1] = 1234;",['a'])        
        assert sys.getrefcount(a[1]) == 2                
        assert a[1] == 1234
    def check_set_double(self,level=5):
        a = UserList([1,2,3])            
        inline_tools.inline("a[1] = 123.0;",['a'])
        assert sys.getrefcount(a[1]) == 2       
        assert a[1] == 123.0        
    def check_set_char(self,level=5):
        a = UserList([1,2,3])            
        inline_tools.inline('a[1] = "bubba";',['a'])
        assert sys.getrefcount(a[1]) == 2       
        assert a[1] == 'bubba'
    def check_set_string(self,level=5):
        a = UserList([1,2,3])            
        inline_tools.inline('a[1] = std::string("sissy");',['a'])
        assert sys.getrefcount(a[1]) == 2       
        assert a[1] == 'sissy'
    def check_set_string(self,level=5):
        a = UserList([1,2,3])            
        inline_tools.inline('a[1] = std::complex<double>(1,1);',['a'])
        assert sys.getrefcount(a[1]) == 2       
        assert a[1] == 1+1j

from UserDict import UserDict
class test_object_set_item_op_key(unittest.TestCase):
    def check_key_refcount(self,level=5):
        a = UserDict()
        code =  """
                py::object one = 1;
                py::object two = 2;
                py::tuple ref_counts(3);
                py::tuple obj_counts(3);
                py::tuple val_counts(3);
                py::tuple key_counts(3);
                obj_counts[0] = a.refcount();
                key_counts[0] = one.refcount();
                val_counts[0] = two.refcount();
                a[1] = 2;
                obj_counts[1] = a.refcount();
                key_counts[1] = one.refcount();
                val_counts[1] = two.refcount();
                a[1] = 2;
                obj_counts[2] = a.refcount();
                key_counts[2] = one.refcount();
                val_counts[2] = two.refcount();
                
                ref_counts[0] = obj_counts;
                ref_counts[1] = key_counts;
                ref_counts[2] = val_counts;
                return_val = ref_counts;
                """
        obj,key,val = inline_tools.inline(code,['a'])
        assert obj[0] == obj[1] and obj[1] == obj[2]
        assert key[0] + 1 == key[1] and key[1] == key[2]
        assert val[0] + 1 == val[1] and val[1] == val[2]
        
    def check_set_double_exists(self,level=5):
        a = UserDict()   
        key = 10.0     
        a[key] = 100.0
        inline_tools.inline('a[key] = 123.0;',['a','key'])
        first = sys.getrefcount(key)
        inline_tools.inline('a[key] = 123.0;',['a','key'])
        second = sys.getrefcount(key)
        assert first == second
        # !! I think the following should be 3
        assert sys.getrefcount(key) ==  5
        assert sys.getrefcount(a[key]) == 2       
        assert a[key] == 123.0
    def check_set_double_new(self,level=5):
        a = UserDict()        
        key = 1.0
        inline_tools.inline('a[key] = 123.0;',['a','key'])
        assert sys.getrefcount(key) == 4 # should be 3       
        assert sys.getrefcount(a[key]) == 2       
        assert a[key] == 123.0
    def check_set_complex(self,level=5):
        a = UserDict()
        key = 1+1j            
        inline_tools.inline("a[key] = 1234;",['a','key'])
        assert sys.getrefcount(key) == 3       
        assert sys.getrefcount(a[key]) == 2                
        assert a[key] == 1234
    def check_set_char(self,level=5):
        a = UserDict()        
        inline_tools.inline('a["hello"] = 123.0;',['a'])
        assert sys.getrefcount(a["hello"]) == 2       
        assert a["hello"] == 123.0
        
    def check_set_class(self,level=5):
        a = UserDict()        
        class foo:
            def __init__(self,val):
                self.val = val
            def __hash__(self):
                return self.val
        key = foo(4)
        inline_tools.inline('a[key] = "bubba";',['a','key'])
        first = sys.getrefcount(key)       
        inline_tools.inline('a[key] = "bubba";',['a','key'])
        second = sys.getrefcount(key)       
        # I don't think we're leaking if this is true
        assert first == second  
        # !! BUT -- I think this should be 3
        assert sys.getrefcount(key) == 4 
        assert sys.getrefcount(a[key]) == 2       
        assert a[key] == 'bubba'
    def check_set_from_member(self,level=5):
        a = UserDict()        
        a['first'] = 1
        a['second'] = 2
        inline_tools.inline('a["first"] = a["second"];',['a'])
        assert a['first'] == a['second']

if __name__ == "__main__":
    ScipyTest('weave.scxx').run()



#
# Title: Provides ParallelExec to execute commands in
#        other (background or parallel) threads.
# Author: Pearu Peteson <pearu@cens.ioc.ee>
# Created: October, 2003
#

__all__ = ['ParallelExec']

import sys
import threading
import Queue
import traceback
import types
import inspect
import time
import atexit

class ParallelExec(threading.Thread):
    """ Create a thread of parallel execution.
    """
    def __init__(self):
        threading.Thread.__init__(self)
        self.__queue = Queue.Queue(0)
        self.__frame = sys._getframe(1)
        self.setDaemon(1)
        self.start()

    def __call__(self,code,frame=None,wait=0):
        """ Execute code in parallel thread inside given frame (default
        frame is where this instance was created).
        If wait is True then __call__ returns after code is executed,
        otherwise code execution happens in background.
        """
        if wait:
            wait_for_code = threading.Event()
        else:
            wait_for_code = None
        self.__queue.put((code,frame,wait_for_code))
        if wait:
            wait_for_code.wait()

    def shutdown(self):
        """ Shutdown parallel thread."""
        self.__queue.put((None,None,None))

    def run(self):
        """ Called by threading.Thread."""
        while 1:
            code, frame, wait_for_code = self.__queue.get()
            if code is None:
                break
            if frame is None:
                frame = self.__frame
            try:
                exec (code, frame.f_globals,frame.f_locals)
            except Exception:
                try:
                    traceback.print_exc()
                except AttributeError:
                    pass
            if wait_for_code is not None:
                wait_for_code.set()



# Python 2.3 distutils.log backported to Python 2.1.x, 2.2.x

import sys

def _fix_args(args,flag=1):
    if type(args) is type(''):
        return args.replace('%','%%')
    if flag and type(args) is type(()):
        return tuple([_fix_args(a,flag=0) for a in args])
    return args

if sys.version[:3]>='2.3':
    from distutils.log import *
    from distutils.log import Log as old_Log
    from distutils.log import _global_log
    class Log(old_Log):
        def _log(self, level, msg, args):
            if level>= self.threshold:
                if args:
                    print _global_color_map[level](msg % _fix_args(args))
                else:
                    print _global_color_map[level](msg)
                sys.stdout.flush()
    _global_log.__class__ = Log

else:
    exec """
# Here follows (slightly) modified copy of Python 2.3 distutils/log.py

DEBUG = 1
INFO = 2
WARN = 3
ERROR = 4
FATAL = 5
class Log:

    def __init__(self, threshold=WARN):
        self.threshold = threshold

    def _log(self, level, msg, args):
        if level >= self.threshold:
            if args:
                print _global_color_map[level](msg % _fix_args(args))
            else:
                print _global_color_map[level](msg)
            sys.stdout.flush()

    def log(self, level, msg, *args):
        self._log(level, msg, args)

    def debug(self, msg, *args):
        self._log(DEBUG, msg, args)

    def info(self, msg, *args):
        self._log(INFO, msg, args)
    
    def warn(self, msg, *args):
        self._log(WARN, red_text(msg), args)
    
    def error(self, msg, *args):
        self._log(ERROR, msg, args)
    
    def fatal(self, msg, *args):
        self._log(FATAL, msg, args)

_global_log = Log()
log = _global_log.log
debug = _global_log.debug
info = _global_log.info
warn = _global_log.warn
error = _global_log.error
fatal = _global_log.fatal

def set_threshold(level):
    _global_log.threshold = level


"""

def set_verbosity(v):
    if v<0:
        set_threshold(ERROR)
    elif v == 0:
        set_threshold(WARN)
    elif v == 1:
        set_threshold(INFO)
    elif v >= 2:
        set_threshold(DEBUG)

from misc_util import red_text, yellow_text, cyan_text
_global_color_map = {
    DEBUG:cyan_text,
    INFO:yellow_text,
    WARN:red_text,
    ERROR:red_text,
    FATAL:red_text
}

set_verbosity(1)



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# The following is so I know about arrays:
from Numeric import *
from scipy_base.fastumath import *
from shapetest import *
from surface import *
from graftypes import *
from plane import *
from slice3 import *
 
class Mesh3d ( Surface ) :
   """m = Mesh3d ( <keyword arguments> ) will create a mesh with
   coordinates and various other characteristics. The keywords are
   as follows:
      For all meshes:
         color_card = <value> specifies which color card you wish
            to use, e. g., "rainbowhls" (the default), "random",
            etc. Although a characteristic of a Graph2d, it can
            be a Surface characteristic since 'link'ed surfaces
            can have different color cards.
            This is not implemented for Gist. Instead, Gist
            allows the option of a "split palette", which causes
            isosurfaces to be shaded and other surfaces to be
            colored by a subset of the current (or specified)
            palette.
         opt_3d = <value> where <value> is a string or a sequence
            of strings giving the 3d or 4d surface characteristics.
            A surface is colored by height in z if a 3d option is
            specified, and by the value of the function c if a 4d
            option is specified. With a wire grid option, the grid
            is colored; with a flat option, the quadrilaterals set
            off by grid points are colored; with a smooth option,
            the surface itself is colored by height; and with an iso
            option, the contour lines are colored. flat and iso options
            may be used together in any combination. wire grid options
            are independent of the other options. Legal arguments for
            set_3d_options are:
            'wm'--monochrome wire grid; 'w3' and 'w4'--3d and 4d
            coloring of wire grid.
            'f3' and 'f4'--flat 3d and 4d coloring options.
            'i3' and 'i4'--3d and 4d isoline (contour line) options.
            's3' and 's4'--3d and 4d smooth coloring options.
            For Gist plots, currently only 'wm', 'f3', and 'f4'
            are currently available. If others are specified, an
            intelligent guess will be made.
         mesh_type = <string> in one of the wire modes, tells what
            form the wire grid takes: "x"--x lines only; "y": y lines only;
            "xy": both a lines and y lines.
            Not implemented for Gist plots.
         mask = <string> controls hidden line removal. Allowed values
            are "none" : transparent graph; "min": simple
            masking; "max" : better masking; "sort": slowest but
            most sophisticated.
            Only the "none" and "sort" are available for Gist.
         z_c_switch = 0 or 1 : set to 1 means switch z and c in the plot.
         z_contours_scale, c_contours_scale = "lin" or "log"
         z_contours_array, c_contours_array = actual array of numbers
            to use for contours
         number_of_z_contours, number_of_c_contours = <integer>
            specifies how many contours to use; they will be computed
            automatically based on the data.
         The last six options are not available in Gist.
      For nonstructured meshes only:
         x = <values> , y = <values> , z = <values> three vectors
            of equal lengths giving the coordinates of the nodes of
            a nonstructured mesh.
         c = <values> , a vector of the same size as x, y, and z
            giving a data value at each of the points specified.
            c could also be an array of such vectors, when
            isosurfaces of more than one function are to be plotted.
         avs = 0 or 1: if 1, the input data represents a nonstructured
            mesh in a sort of AVS format, which will be explained in
            more detail below. The data will be translated into the
            Narcisse format prior to being sent to Narcisse.
         cell_descr = <integer array> if present, this keyword signifies
            a nonstructured mesh submitted in the Narcisse format,
            as explained in the Narcisse manual. For nonstructured
            mesh data, one of these two keywords must be present.
         In the case avs = 1, at least one of the following four keywords
            (hex, tet, prism, or pyr) must be present also:
         hex = [ <list of hexahedral cell data> ] The entries in the
            list (in order) must be:
            an integer number n_zones, which is the number of hex
            cells in the mesh; and a matrix nz whose dimensions are
            n_zones by 8; nz [i][0], nz [i][1], ... nz [i][7] give
            the indices of the 8 vertices of the ith zone in canonical
            order (one side in the outward normal direction, then the
            corresponding vertices of the opposite side in the inward
            normal direction).
         tet = [ <list of tetrahedral cell data> ] The list is the
            same format as for hex data. The matrix nz will be
            n_zones by 4, and each row gives the indices of the
            apex and then the base in inward normal order.
         prism = [ <list of prismatic cell data> ] The list is the
            same format as for hex data. The matrix nz will be
            n_zones by 6, and each row gives the indices of one
            triangular side in the outward normal direction, then
            the corresponding vertices of the opposite side in the
            inward normal direction.
         pyr = [ <list of pyramidal cell data> ] The list is the
            same format as for hex data. The matrix nz will be
            n_zones by 5, and each row gives the indices of the
            apex and then the base in inward normal order.
         In the case of avs = 0, the following keywords are all
            required:
         cell_descr = <values> , a vector of integers giving the
            description of the cells of the mesh, as specified in
            the Narcisse manual.
         no_cells = <integer value> , the number of cells in the mesh.
            (Here cells refers to the number of faces of cells).
      For structured meshes only:
         x = <values> , y = <values> , z = <values> three vectors
            giving the coordinates of the nodes of a structured
            rectangular mesh.
         c = <values> , a three-dimensional array whose [i, j, k]
            element gives the associated data value at (x [i], y [j],
            z [k]). Could also be one less in each direction,
            giving a cell-centered quantity.
            c could also be an array of such arrays, when
            isosurfaces of more than one function are to be plotted.
   """

   _MeshInitError = "MeshInitError"

   def type (self) :
       return Mesh3dType

   def __init__ ( self , *kwds , ** keywords ) :
       if len (kwds) == 1 :
          keywords = kwds[0]
       self.cell_dict = {}
       if keywords.has_key ("hex") :
          self.cell_dict ["hex"] = keywords ["hex"]
       if keywords.has_key ("tet") :
          self.cell_dict ["tet"] = keywords ["tet"]
       if keywords.has_key ("prism") :
          self.cell_dict ["prism"] = keywords ["prism"]
       if keywords.has_key ("pyr") :
          self.cell_dict ["pyr"] = keywords ["pyr"]
       if (keywords.has_key ("avs") and keywords ["avs"] == 0) or \
          (not keywords.has_key ("avs")
           and not keywords.has_key ("cell_descr")) :
          # structured case works basically just like a Surface
          Surface.__init__ ( self, keywords )
          self.structured = 1
       else : # nonstructured case, at least initialize the generics
          self.structured = 0
          self.generic_init ( keywords )
          if keywords.has_key ("avs") :
             self.avs = keywords ["avs"]
          else :
             self.avs = None
          if not keywords.has_key ("x") or \
             not keywords.has_key ("y") or \
             not keywords.has_key ("z") or \
             not keywords.has_key ("c") :
             raise self._MeshInitError , \
                "x, y, z, and c keywords required for nonstructured mesh."
          self.x = keywords ["x"]
          self.y = keywords ["y"]
          self.z = keywords ["z"]
          self.c = keywords ["c"]
          if keywords.has_key ("cell_descr" ) : # already there
             if not keywords.has_key ("no_cells") :
                raise self._MeshInitError , \
                   "Keyword no_cells is required if avs is not specified."
             self.cell_descr = keywords ["cell_descr"]
             self.number_of_cells = keywords ["no_cells"]
          elif not keywords.has_key ("avs") :
             raise self._MeshInitError , \
                "keywords must include avs = 1 or else cell_descr = values."

   # Conversion tables for AVS to Narcisse formats

   _cell_types = ["tet", "pyr", "prism", "hex"]
   _no_of_vertices = { "hex" : 8, "tet" : 4, "pyr" : 5, "prism" : 6 }
   _no_of_rect_faces = { "hex" : 6, "tet" : 0, "pyr" : 1, "prism" : 3 }
   _no_of_tri_faces = { "hex" : 0, "tet" : 4, "pyr" : 4, "prism" : 2 }
   # The next few items give the standard numbering of the faces of
   # the various cells.
   _hex_triangular_faces = [ ]
   _tet_triangular_faces = [ [0, 1, 2], [0, 3, 1], [0, 2, 3], [1, 3, 2]]
   _pyr_triangular_faces = [ [0, 1, 2], [0, 4, 1], [0, 3, 4], [0, 2, 3]]
   _prism_triangular_faces = [ [0, 1, 2], [3, 5, 4]]
   _hex_rectangular_faces = [ [0, 1, 2, 3], [1, 0, 4, 5], [2, 1, 5, 6],
                              [3, 2, 6, 7], [0, 3, 7, 4], [4, 7, 6, 5]]
   _tet_rectangular_faces = [ ]
   _pyr_rectangular_faces = [ [1, 4, 3, 2]]
   _prism_rectangular_faces = [ [0, 3, 4, 1], [1, 4, 5, 2], [2, 5, 3, 0]]
   _tri_face_numbers = { "hex" : _hex_triangular_faces , 
                         "tet" : _tet_triangular_faces ,
                         "pyr" : _pyr_triangular_faces ,
                         "prism" : _prism_triangular_faces }
   _rect_face_numbers = { "hex" : _hex_rectangular_faces ,
                          "tet" : _tet_rectangular_faces ,
                          "pyr" : _pyr_rectangular_faces ,
                          "prism" : _prism_rectangular_faces }

   def get_verts_list (self) :
      """get_verts_list () returns a list of vertex arrays in the
      form wanted by mesh3. Note: if they are in avs order,
      they need to be put into Gist order (pyramids and tets
      are OK, only prisms and hexahedra need vertices permuted).
      """
      verts = []
      for k in self._cell_types :
         if self.cell_dict.has_key (k) :
            n_z = self.cell_dict[k][1]
            if n_z.shape [1] == 8 :
               verts.append (
                  take (n_z, array ( [0, 1, 3, 2, 4, 5, 7, 6])))
            elif n_z.shape [1] == 6 :
               verts.append (
                  take (n_z, array ( [3, 0, 4, 1, 5, 2])))
            else :
               verts.append (n_z)
      if len (verts) == 1 :
         verts = verts [0]
      return verts

   def create_Narcisse_format ( self ) :
      """create_Narcisse_format ( <keyword dict> ) is an internal
      routine which takes input in (essentially) avs format and
      converts it into the format desired by Narcisse.
      """
      if hasattr (self, "cell_descr") : # don't calculate it again.
         return
      if not self.cell_dict.has_key ("hex") \
         and not self.cell_dict.has_key ("tet") \
         and not self.cell_dict.has_key ("prism") \
         and not self.cell_dict.has_key ("pyr") :
         raise _MeshInitError, \
            "In avs mode, at least one of the keywords " + \
            "(hex, tet, pyr, prism) is required."
      n_zones = { }
      nz = { }
      for k in self._cell_types :
         if self.cell_dict.has_key (k) :
            n_zones [k] = self.cell_dict [k][0]
            nz [k] = self.cell_dict[k][1]
         else :
            n_zones [k] = 0
            nz [k] = [ ]
      self.cell_descr = zeros (30 * n_zones ["hex"] +
                               21 * n_zones ["pyr"] +
                               16 * n_zones ["tet"] +
                               23 * n_zones ["prism"], Int)
      self.number_of_cells = n_zones ["hex"] * 6 + n_zones ["pyr"] * 5 + \
                             n_zones ["tet"] * 4 + n_zones ["prism"] * 5
      n = 0
      start = 0
      # Fill the start of the cell_descriptor array
      # The first part is simply a sequence of indices into the second
      # part of the self.cell_descr array; Each entry tells where the indices
      # for this particular face begin.
      cell_j = 0
      for k in self._cell_types :
         nrf = self._no_of_rect_faces [k]
         ntf = self._no_of_tri_faces [k]
         for i in range (start, start + n_zones [k]) :
            for j in range (ntf) : # count triangular faces first
               n = n + 3
               self.cell_descr [cell_j] = n
               cell_j = cell_j + 1
            for j in range (nrf) : # count rectangles next
               n = n + 4
               self.cell_descr [cell_j] = n
               cell_j = cell_j + 1
         start = start + n_zones [k]
      # Next we go through each type of cell and enter the indices
      # of its faces, face-by-face, triangular sides first by convention.
      for k in self._cell_types :
         nrf = self._no_of_rect_faces [k]
         ntf = self._no_of_tri_faces [k]
         n_z = nz [k]
         for i in range (start, start + n_zones [k]) :
            n_z_ = n_z [i - start]
            for j in range (ntf) : # do triangular faces first
               ind = self._tri_face_numbers [k][j]
               for l in range (3) :
                  self.cell_descr [cell_j + l] = n_z_ [ind [l]]
               cell_j = cell_j + 3
            for j in range (nrf) : # do rectangular faces next
               ind = self._rect_face_numbers [k][j]
               for l in range (4) :
                  self.cell_descr [cell_j + l] = n_z_ [ind [l]]
               cell_j = cell_j + 4
         start = start + n_zones [k]

   def new ( self , ** keywords ) :
      """new (...keyword arguments...) allows you to reuse a
      previously defined mesh.
      """
      del self.x, self.y, self.z, self.c, self.color_card, self.opt_3d, \
          self.mask, self.z_c_switch, self.z_contours_scale, \
          self.c_contours_scale, self.z_contours_array, \
          self.c_contours_array, self.number_of_z_contours, \
          self.number_of_c_contours, self.mesh_type
      if not self.structured :
         del self.cell_descr, self.number_of_cells
      self.__init__ ( keywords )

from plane import *

class Slice :
   """
   Class Slice is used to contain a geometric description of
   an isosurface or plane Slice of a Mesh3d object. It is 
   created by the builtin function sslice, defined later in
   this module.
   Slice is created as follows:
       Slice (nv, xyzv, [, val [, plane [, iso [, opt_3d]]]])
   where :
     nv is a one-dimensional integer array whose ith entry is the
        number of vertices of the ith face.
     xyzv is a two-dimensional array dimensioned sum (nv) by 3.
        The first nv [0] triples are the vertices of face [0],
        the next nv [1] triples are the vertices of face [1], etc.
     val (if present) is an array the same length as nv whose
        ith entry specifies a color for face i.
     plane (if present) says that this is a plane Slice, and
        all the vertices xyzv lie in this plane.
     iso (if present) says that this is the isosurface for
        the given value.
     opt_3d (if present) specifies the 3d options used to
        color the slice.
     mask (if present) specifies how the surface is to be masked.
     contours (if present) is either an array of contour values
        or an integer giving the number of contours.
     scale (if present) is the contour scale: "lin", "log",
        or "normal".
     edges (if present) is 1 for showing edges, 0 for not.
   A Slice object or two Slice objects are created by a call
   to the function sslice (q. v.). The function sslice accepts
   either a mesh and a specification of how to slice it
   (isosurface or plane), or else a previously created Slice,
   a plane to slice it with, and whether you want to keep
   the resulting "front" Slice or both Slices.
   """

   def type (self) :
       return Slice3dType

   def __init__ (self, nv, xyzv, val = None, plane = None, iso = None,
       smooth = None, opt_3d = ["wm", "f3"], mask = "max", contours = None,
       scale = "lin", edges = 0) :
       self.nv = nv
       self.xyzv = xyzv
       self.val = val
       self.plane = plane
       self.iso = iso
       self.smooth = smooth
       if type (opt_3d) != ListType :
          self.opt_3d = [opt_3d]
       else :
          self.opt_3d = opt_3d
       self.mask = mask
       self.contours = contours
       self.scale = scale
       self.edges = edges
       self.mesh_type = "xy"
       self.z_c_switch = 0
       self.z_contours_scale = scale
       self.c_contours_scale = scale
       if type (self.contours) == IntType or self.contours is None :
          self.number_of_z_contours = self.contours
          self.number_of_c_contours = self.contours
          self.z_contours_array = None
          self.c_contours_array = None
       elif type (self.contours) == ArrayType :
          self.z_contours_array = self.contours
          self.c_contours_array = self.contours
          self.number_of_z_contours = len (self.contours)
          self.number_of_c_contours = len (self.contours)
       if edges == 0 :
          self.edges = "w3" in self.opt_3d or \
                       "w4" in self.opt_3d or \
                       "wm" in self.opt_3d

   def __del__ (self) :
       del self.nv
       del self.xyzv
       if self.val is not None: del self.val
       if self.plane is not None: del self.plane
       del self.mask
       if self.iso is not None: del self.iso
       if self.smooth is not None: del self.smooth
       del self.opt_3d
       if self.contours is not None: del self.contours
       del self.scale
       del self.edges

   def new (self, nv, xyzv, val = None, plane = None, iso = None,
      smooth = None, opt_3d = ["wm", "f3"], mask = "max", contours = None,
      scale = "lin", edges = 0) :
       del self.nv
       del self.xyzv
       del self.val
       del self.plane
       del self.mask
       del self.iso
       del self.smooth
       del self.opt_3d
       del self.contours
       del self.scale
       del self.edges
       self.__init__ (nv, xyzv, val, plane, iso, smooth, opt_3d,
          mask, contours, scale, edges)

   def set ( self , ** keywords ) :
       """ set (...keyword arguments...) allows you to set individual
       Slice characteristics. No error checking is done.
       """
       for k in keywords.keys ():
          setattr (self, k, keywords [k])
       if type (self.opt_3d) != ListType :
          self.opt_3d = [self.opt_3d]
       if self.edges == 0 :
          self.edges = "w3" in self.opt_3d or \
                       "w4" in self.opt_3d or \
                       "wm" in self.opt_3d

_SliceError = "SliceError"

def sslice ( v1, v2, varno = 1, nslices = 1, opt_3d = None) :
    """This function slices a Mesh3d or a previous Slice object,
    and returns a new Slice object, depending on how it is called.
       sslice (m, plane, varno = 1)
    returns a plane Slice through the specified mesh m. varno
    is the number of the variable to be used to color the Slice
    (1-based).
       sslice (m, val, varno = 1)
    returns an isosurface Slice of m. varno is the number of
    the variable with respect to which the Slice is taken (1-based)
    and val is its value on the isosurface. varno can be let default
    to 1 if there is only one function to worry about.
       sslice (s, plane, nslices = 1)
    slices the isosurface s and returns the part in "front" of the
    plane if nslices == 1, or returns a pair of Slices [front, back]
    if nslices == 2. If s is a Surface, then this operation will
    produce a Slice or pair of Slices from the given Surface.
    N. B. If you want just the "back" surface, you can achieve this
    by calling sslice with - plane instead of plane.
       The opt_3d argument can be used to overrule defaults, which
    are intelligently figured out, but may not always be what you want.
    """
    if v1.type () == Mesh3dType :
       if type (v1.c) != ListType :
          funcs = [v1.c]
       else :
          funcs = v1.c
       if not hasattr (v2, "type") or v2.type () != PlaneType :
          try :
             val = float (v2)
          except :
             raise _SliceError, \
                "second argument is not coercible to float."
          plane = None
       else :
          plane = v2
          val = None
       if opt_3d is None :
          if plane is not None:
             opt_3d = ["wm", "f4"]
          else :
             opt_3d = ["f4"]

       # First we build a "mesh3" if v1 does not already have one.
       if v1.structured :           #and not hasattr (v1, "m3") :
          setattr (v1, "m3", mesh3 (v1.x, v1.y, v1.z, funcs = funcs))
       elif not v1.structured and v1.avs == 1 : #and not hasattr (v1, "m3") :
          if no_of_dims (v1.x) == 3 and \
             no_of_dims (v1.y) == 3 and \
             no_of_dims (v1.z) == 3 :
             setattr (v1, "m3", mesh3 (array ( [v1.x, v1.y, v1.z], Float),
                funcs = funcs))
          elif no_of_dims (v1.x) == 1 and \
             no_of_dims (v1.y) == 1 and \
             no_of_dims (v1.z) == 1 :
             setattr (v1, "m3", mesh3 (v1.x, v1.y, v1.z, funcs = funcs,
                verts = v1.get_verts_list ()))
          else :
             raise _SliceError, \
                "x, y, and z must have either 1 or 3 dimensions."
       elif not hasattr (v1, "m3") :
          raise _SliceError, \
             "Option not implemented."
       # A Yorick 'mesh3' object now exists, slice it as specified.
       if plane is None : # An isosurface Slice
          [nv, xyzv, dum] = slice3 (v1.m3, varno, None, None, value = val)
          return Slice (nv, xyzv, iso = val, opt_3d = opt_3d)
       else :             # a plane Slice
          [nv, xyzv, val] = slice3 (v1.m3, plane.rep (), None, None, varno )
          return Slice (nv, xyzv, val, plane = plane, opt_3d = opt_3d)
    elif v1.type () == SurfaceType :
       plane = v2
       if not hasattr (v2, "type") or v2.type () != PlaneType :
          raise _SliceError, \
             "Second argument is not a Plane."

       if v1.z_c_switch :
          z = v1.c
          c = v1.z
          if "s3" in v1.opt_3d :
             scale = v1.c_contours_scale
             if v1.c_contours_array is not None:
                contours = v1.c_contours_array
             else :
                contours = v1.number_of_c_contours
          elif "s4" in v1.opt_3d :
             scale = v1.z_contours_scale
             if v1.z_contours_array is not None:
                contours = v1.z_contours_array
             else :
                contours = v1.number_of_z_contours
       else :
          z = v1.z
          c = v1.c
          if "s3" in v1.opt_3d :
             scale = v1.z_contours_scale
             if v1.z_contours_array is not None:
                contours = v1.z_contours_array
             else :
                contours = v1.number_of_z_contours
          elif "s4" in v1.opt_3d :
             scale = v1.c_contours_scale
             if v1.c_contours_array is not None:
                contours = v1.c_contours_array
             else :
                contours = v1.number_of_c_contours
       edges = "w3" in v1.opt_3d or "w4" in v1.opt_3d or "wm" in v1.opt_3d \
            or "i3" in v1.opt_3d or "i4" in v1.opt_3d
       if "s4" in v1.opt_3d :
          [nv, xyzv, col] = slice3mesh (v1.x, v1.y, z, color = c, smooth = 1)
       elif "f3" in v1.opt_3d :
          [nv, xyzv, col] = slice3mesh (v1.x, v1.y, z, color = z, smooth = 0)
          contours = None
          scale = "lin"
       elif "f4" in v1.opt_3d :
          [nv, xyzv, col] = slice3mesh (v1.x, v1.y, z, color = c, smooth = 0)
          contours = None
          scale = "lin"
       else :
          [nv, xyzv, col] = slice3mesh (v1.x, v1.y, z)
          contours = None
          scale = "lin"
       if nslices == 1 :                                    
          [nv, xyzv, col] = slice2 (plane.rep (), nv, xyzv, col)
          return Slice (nv, xyzv, col, opt_3d = v1.opt_3d, mask = v1.mask,
             contours = contours, scale = scale, edges = edges)
       elif nslices == 2 :
          [nv, xyzv, col, nb, xyzb, valb] = slice2x (plane.rep (),
              nv, xyzv, col)
          return [Slice (nv, xyzv, col, opt_3d = v1.opt_3d, mask = v1.mask,
             contours = contours, scale = scale, edges = edges),
                  Slice (nb, xyzb, valb, opt_3d = v1.opt_3d, mask = v1.mask,
             contours = contours, scale = scale, edges = edges)]
       else :
          raise _SliceError, "Illegal number (" + `nslices` + \
             ") of slices requested."
    elif v1.type () == Slice3dType :
       plane = v2
       if not hasattr (v2, "type") or v2.type () != PlaneType :
          raise _SliceError, \
             "Second argument is not a Plane."

       if nslices == 1 :
          [nv, xyzv, val] = slice2 (plane.rep (), v1.nv, v1.xyzv,
              v1.val)
          return Slice (nv, xyzv, val, plane = v1.plane, iso = v1.iso,
                        opt_3d = v1.opt_3d)
       elif nslices == 2 :
          [nf, xyzf, valf, nb, xyzb, valb] = slice2x (plane.rep (), v1.nv,
              v1.xyzv, v1.val)
          return [Slice (nf, xyzf, valf, plane = v1.plane, iso = v1.iso,
                         opt_3d = v1.opt_3d),
                  Slice (nb, xyzb, valb, plane = v1.plane, iso = v1.iso,
                         opt_3d = v1.opt_3d)]
       else :
          raise _SliceError, "Illegal number (" + `nslices` + \
             ") of slices requested."
    else :
       raise _SliceError, "Can only slice a Mesh3d or another Slice."



import unittest
from Numeric import *
# The following try/except so that non-SciPy users can still use blitz
try:
    from scipy_base.fastumath import *
except:
    pass # scipy_base.fastumath not available    
import RandomArray
import time

from scipy_test.testing import *
set_package_path()
from weave import ast_tools
restore_path()

set_local_path()
from weave_test_utils import *
restore_path()

class test_harvest_variables(unittest.TestCase):
    """ Not much testing going on here, but 
        at least it is a flame test.
    """    
    def generic_test(self,expr,desired):
        import parser
        ast_list = parser.suite(expr).tolist()
        actual = ast_tools.harvest_variables(ast_list)
        print_assert_equal(expr,actual,desired)

    def check_simple_expr(self):
        """convert simple expr to blitz
           
           a[:1:2] = b[:1+i+2:]
        """
        expr = "a[:1:2] = b[:1+i+2:]"        
        desired = ['a','b','i']        
        self.generic_test(expr,desired)

if __name__ == "__main__":
    ScipyTest('weave.ast_tools').run()




import os
import sys

from distutils.command.sdist import *
from distutils.command.sdist import sdist as old_sdist
from scipy_distutils import log
from scipy_distutils import line_endings

class sdist(old_sdist):
    def add_defaults (self):
        old_sdist.add_defaults(self)

        if self.distribution.has_data_files():
            self.filelist.extend(self.distribution.get_data_files())

    def make_release_tree (self, base_dir, files):
        """Create the directory tree that will become the source
        distribution archive.  All directories implied by the filenames in
        'files' are created under 'base_dir', and then we hard link or copy
        (if hard linking is unavailable) those files into place.
        Essentially, this duplicates the developer's source tree, but in a
        directory named after the distribution, containing only the files
        to be distributed.
        """
        # Create all the directories under 'base_dir' necessary to
        # put 'files' there; the 'mkpath()' is just so we don't die
        # if the manifest happens to be empty.
        
        dest_files = remove_common_base(files)
        self.mkpath(base_dir)
        dir_util.create_tree(base_dir, dest_files,
                             verbose=self.verbose, dry_run=self.dry_run)

        # And walk over the list of files, either making a hard link (if
        # os.link exists) to each one that doesn't already exist in its
        # corresponding location under 'base_dir', or copying each file
        # that's out-of-date in 'base_dir'.  (Usually, all files will be
        # out-of-date, because by default we blow away 'base_dir' when
        # we're done making the distribution archives.)

        
    
        if 0 and hasattr(os, 'link'):        # can make hard links on this system
            link = 'hard'
            msg = "making hard links in %s..." % base_dir
        else:                           # nope, have to copy
            link = None
            msg = "copying files to %s..." % base_dir
        self._use_hard_link = not not link

        if not files:
            log.warn("no files to distribute -- empty manifest?")
        else:
            log.info(msg)
        
        dest_files = [os.path.join(base_dir,file) for file in dest_files]
        file_pairs = zip(files,dest_files)
        for file,dest in file_pairs:
            if not os.path.isfile(file):
                log.warn("'%s' not a regular file -- skipping", file)
            else:
                #ej: here is the only change -- made to handle
                # absolute paths to files as well as relative
                #par,file_name = os.path.split(file)
                #dest = os.path.join(base_dir, file_name)
                # end of changes
                
                # old code
                #dest = os.path.join(base_dir, file)
                #end old code
                self.copy_file(file, dest, link=link)

        self.distribution.metadata.write_pkg_info(base_dir)
        #raise ValueError
    # make_release_tree ()

    def make_distribution (self):
        """ Overridden to force a build of zip files to have Windows line 
            endings and tar balls to have Unix line endings.
            
            Create the source distribution(s).  First, we create the release
            tree with 'make_release_tree()'; then, we create all required
            archive files (according to 'self.formats') from the release tree.
            Finally, we clean up by blowing away the release tree (unless
            'self.keep_temp' is true).  The list of archive files created is
            stored so it can be retrieved later by 'get_archive_files()'.
        """
        # Don't warn about missing meta-data here -- should be (and is!)
        # done elsewhere.
        base_dir = self.distribution.get_fullname()
        base_name = os.path.join(self.dist_dir, base_dir)
        files = map(os.path.abspath, self.filelist.files)
        self.make_release_tree(base_dir, files)
        archive_files = []              # remember names of files we create
        for fmt in self.formats:
            modified_files,restore_func = self.convert_line_endings(base_dir,fmt)
            file = self.make_archive(base_name, fmt, base_dir=base_dir)
            archive_files.append(file)
            if self._use_hard_link:
                map(restore_func,modified_files)

        self.archive_files = archive_files

        if not self.keep_temp:
            dir_util.remove_tree(base_dir, self.verbose, self.dry_run)

    def convert_line_endings(self,base_dir,fmt):
        """ Convert all text files in a tree to have correct line endings.
            
            gztar --> \n   (Unix style)
            zip   --> \r\n (Windows style)
        """
        if fmt == 'gztar':
            return line_endings.dos2unix_dir(base_dir),line_endings.unix2dos
        elif fmt == 'zip':
            return line_endings.unix2dos_dir(base_dir),line_endings.dos2unix
        return [],lambda a:None

def remove_common_base(files):
    """ Remove the greatest common base directory from all the
        absolute file paths in the list of files.  files in the
        list without a parent directory are not affected.
    """
    rel_files = filter(lambda x: not os.path.dirname(x),files)
    abs_files = filter(os.path.dirname,files)
    base = find_common_base(abs_files)
    # will leave files with local path unaffected
    # and maintains original file order
    results = [file[len(base):] for file in files]
    return results

def find_common_base(files):
    """ Find the "greatest common base directory" of a list of files
    """
    if not files:
        return ''
    result = ''
    d,f = os.path.split(files[0])
    keep_looking = 1    
    while(keep_looking and d):
        keep_looking = 0
        for file in files:
            if string.find('start'+file,'start'+d) == -1:
                keep_looking = 1
                break
        if keep_looking:
            d,f = os.path.split(d)
        else:
            result = d
            
    if d: 
        d = os.path.join(d,'')
    return d        
